"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("reflect-metadata");
var react_1 = tslib_1.__importStar(require("react"));
var hooks_1 = require("../hooks");
var core_1 = require("../core");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var react_test_renderer_1 = require("react-test-renderer");
var di_1 = require("@asuka/di");
var sinon_1 = tslib_1.__importDefault(require("sinon"));
var ASYNC_DELAY_TIME = 1000;
var GlobalModule = /** @class */ (function (_super) {
    tslib_1.__extends(GlobalModule, _super);
    function GlobalModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = {
            user: '',
        };
        return _this;
    }
    GlobalModule.prototype.setUser = function (state, payload) {
        state.user = payload;
    };
    GlobalModule.prototype.fetchUser = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(ASYNC_DELAY_TIME), operators_1.map(function () { return _this.getActions().setUser('global'); }));
    };
    tslib_1.__decorate([
        core_1.ImmerReducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, String]),
        tslib_1.__metadata("design:returntype", void 0)
    ], GlobalModule.prototype, "setUser", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], GlobalModule.prototype, "fetchUser", null);
    GlobalModule = tslib_1.__decorate([
        core_1.Module('Global')
    ], GlobalModule);
    return GlobalModule;
}(core_1.Ayanami));
var CountModule = /** @class */ (function (_super) {
    tslib_1.__extends(CountModule, _super);
    function CountModule(globalModule) {
        var _this = _super.call(this) || this;
        _this.globalModule = globalModule;
        _this.defaultState = {
            count: 0,
        };
        return _this;
    }
    CountModule.prototype.setCount = function (state, payload) {
        state.count = payload;
    };
    CountModule.prototype.addCount = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.withLatestFrom(this.state$), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.addCountAsync = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(ASYNC_DELAY_TIME), operators_1.withLatestFrom(this.state$), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.waitUserAddCount = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.combineLatest(this.globalModule.state$.pipe(operators_1.skipWhile(function (v) { return !v.user; }), operators_1.take(1)), this.state$.pipe(operators_1.take(1))), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 3), state = _b[2];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    tslib_1.__decorate([
        core_1.ImmerReducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Number]),
        tslib_1.__metadata("design:returntype", void 0)
    ], CountModule.prototype, "setCount", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], CountModule.prototype, "addCount", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], CountModule.prototype, "addCountAsync", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], CountModule.prototype, "waitUserAddCount", null);
    CountModule = tslib_1.__decorate([
        core_1.Module('CountModule'),
        tslib_1.__metadata("design:paramtypes", [GlobalModule])
    ], CountModule);
    return CountModule;
}(core_1.Ayanami));
describe('Hooks specs: useAyanami', function () {
    var SimpleComponent = function () {
        var _a = tslib_1.__read(hooks_1.useAyanami(CountModule), 2), state = _a[0], actions = _a[1];
        var addCount = react_1.useCallback(function () {
            actions.addCount();
        }, []);
        var addCountAsync = react_1.useCallback(function () {
            actions.addCountAsync();
        }, []);
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("span", null, state.count),
            react_1.default.createElement("button", { id: "add-count", onClick: addCount }, "+"),
            react_1.default.createElement("button", { id: "add-count-async", onClick: addCountAsync }, "+"),
            react_1.default.createElement("button", { id: "set-count", onClick: function () { return actions.setCount(10); } }, "set")));
    };
    var testRenderer;
    var getCount;
    var timer;
    beforeEach(function () {
        timer = sinon_1.default.useFakeTimers();
        testRenderer = react_test_renderer_1.create(react_1.default.createElement(di_1.InjectableContext, null,
            react_1.default.createElement(SimpleComponent, null)));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        timer.restore();
        di_1.rootInjector.reset();
        di_1.rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should render correct state', function () {
        expect(getCount()).toBe(0);
    });
    it('should dispatch action', function () {
        react_test_renderer_1.act(function () {
            testRenderer.root.find(function (instance) { return instance.props.id === 'set-count'; }).props.onClick();
        });
        expect(getCount()).toBe(10);
    });
    it('should dispatch effect', function () {
        expect(getCount()).toBe(0);
        react_test_renderer_1.act(function () {
            testRenderer.root.find(function (instance) { return instance.props.id === 'add-count'; }).props.onClick();
        });
        expect(getCount()).toBe(1);
    });
    it('should dispatch async effect', function () {
        react_test_renderer_1.act(function () {
            testRenderer.root.find(function (instance) { return instance.props.id === 'add-count-async'; }).props.onClick();
            timer.tick(ASYNC_DELAY_TIME);
        });
        expect(getCount()).toBe(1);
    });
    it('should only render once when update state right during rendering', function () {
        var spyFn = jest.fn();
        var TestComponent = function () {
            var _a = tslib_1.__read(hooks_1.useAyanami(CountModule), 2), state = _a[0], actions = _a[1];
            if (state.count === 2) {
                actions.addCount();
            }
            react_1.useEffect(function () {
                spyFn(state.count);
            }, [state.count]);
            return (react_1.default.createElement("div", null,
                react_1.default.createElement("span", null, state.count),
                react_1.default.createElement("button", { id: "set-count", onClick: function () { return actions.setCount(2); } }, "set")));
        };
        var renderer = react_test_renderer_1.create(react_1.default.createElement(TestComponent, null));
        // https://github.com/facebook/react/issues/14050 to trigger useEffect manually
        renderer.update(react_1.default.createElement(TestComponent, null));
        expect(spyFn.mock.calls).toEqual([[0]]);
        react_test_renderer_1.act(function () {
            renderer.root.findByType('button').props.onClick();
        });
        expect(spyFn.mock.calls).toEqual([[0], [3]]);
    });
    it('should be able to use state$ from the other module', function () {
        var TestComponent = function () {
            var _a = tslib_1.__read(hooks_1.useAyanami(CountModule), 2), state = _a[0], actions = _a[1];
            var globalActions = hooks_1.useAyanamiDispatchers(GlobalModule);
            react_1.useEffect(function () {
                actions.waitUserAddCount();
                globalActions.fetchUser();
            }, []);
            return react_1.default.createElement("span", null, state.count);
        };
        var renderer = react_test_renderer_1.create(react_1.default.createElement(TestComponent, null));
        react_test_renderer_1.act(function () {
            renderer.update(react_1.default.createElement(TestComponent, null));
        });
        react_test_renderer_1.act(function () {
            timer.tick(ASYNC_DELAY_TIME);
            renderer.update(react_1.default.createElement(TestComponent, null));
        });
        expect(getCount()).toBe(1);
    });
    it('should be able to catch reducer error', function () {
        var error = new TypeError('whatever');
        var ReducerErrorModule = /** @class */ (function (_super) {
            tslib_1.__extends(ReducerErrorModule, _super);
            function ReducerErrorModule() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {};
                return _this;
            }
            ReducerErrorModule.prototype.throw = function () {
                throw error;
            };
            tslib_1.__decorate([
                core_1.Reducer(),
                tslib_1.__metadata("design:type", Function),
                tslib_1.__metadata("design:paramtypes", []),
                tslib_1.__metadata("design:returntype", void 0)
            ], ReducerErrorModule.prototype, "throw", null);
            ReducerErrorModule = tslib_1.__decorate([
                core_1.Module('ReducerError')
            ], ReducerErrorModule);
            return ReducerErrorModule;
        }(core_1.Ayanami));
        var spy = sinon_1.default.spy();
        var Component = function () {
            var actions = hooks_1.useAyanamiDispatchers(ReducerErrorModule);
            try {
                actions.throw();
            }
            catch (e) {
                spy(e);
            }
            return react_1.default.createElement("div", null);
        };
        react_test_renderer_1.create(react_1.default.createElement(di_1.InjectableContext, null,
            react_1.default.createElement(Component, null)));
        expect(spy.args).toStrictEqual([[error]]);
    });
});
describe('Hooks spec: useAyanami with config', function () {
    var Component = function () {
        var _a = tslib_1.__read(hooks_1.useAyanami(CountModule, {
            selector: function (state) { return state.count; },
            mutateStateOnFirstRendering: function (state) {
                state.count = 2;
            },
        }), 2), count = _a[0], actions = _a[1];
        var addCount = react_1.useCallback(function () {
            actions.addCount();
        }, []);
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("span", null, count),
            react_1.default.createElement("button", { id: "add-count", onClick: addCount }, "+")));
    };
    var testRenderer;
    var getCount;
    beforeEach(function () {
        testRenderer = react_test_renderer_1.create(react_1.default.createElement(Component, null));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        di_1.rootInjector.reset();
        di_1.rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should return selected state from store', function () {
        expect(getCount()).toBe(2);
    });
    it('should only mutate state on first rendering', function () {
        expect(getCount()).toBe(2);
        react_test_renderer_1.act(function () {
            testRenderer.root.findByType('button').props.onClick();
        });
        expect(getCount()).toBe(3);
    });
});
describe('Hooks spec: useAyanamiState', function () {
    var Component = function () {
        var count = hooks_1.useAyanamiState(CountModule, {
            selector: function (state) { return state.count; },
        });
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("span", null, count)));
    };
    var testRenderer;
    var getCount;
    beforeEach(function () {
        testRenderer = react_test_renderer_1.create(react_1.default.createElement(Component, null));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        di_1.rootInjector.reset();
        di_1.rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should return selected state from store', function () {
        expect(getCount()).toBe(0);
    });
});
describe('Hooks spec: useAyanamiDispatchers', function () {
    var Component = function () {
        var state = hooks_1.useAyanamiState(CountModule);
        var actions = hooks_1.useAyanamiDispatchers(CountModule);
        var addCount = react_1.useCallback(function () {
            actions.addCount();
        }, []);
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("span", null, state.count),
            react_1.default.createElement("button", { id: "add-count", onClick: addCount }, "+")));
    };
    var testRenderer;
    var getCount;
    beforeEach(function () {
        testRenderer = react_test_renderer_1.create(react_1.default.createElement(Component, null));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        di_1.rootInjector.reset();
        di_1.rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should return selected state from store', function () {
        expect(getCount()).toBe(0);
    });
    it('should dispatch effect', function () {
        react_test_renderer_1.act(function () {
            testRenderer.root.findByType('button').props.onClick();
        });
        expect(getCount()).toBe(1);
    });
});
