"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("reflect-metadata");
var di_1 = require("@asuka/di");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var sinon_1 = require("sinon");
var testbed_1 = require("../testbed");
var core_1 = require("../core");
var ASYNC_DELAY_TIME = 1000;
var GlobalModule = /** @class */ (function (_super) {
    tslib_1.__extends(GlobalModule, _super);
    function GlobalModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = {
            user: '',
        };
        return _this;
    }
    GlobalModule.prototype.setUser = function (state, payload) {
        state.user = payload;
    };
    GlobalModule.prototype.fetchUser = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(ASYNC_DELAY_TIME), operators_1.map(function () { return _this.getActions().setUser('global'); }));
    };
    tslib_1.__decorate([
        core_1.ImmerReducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, String]),
        tslib_1.__metadata("design:returntype", void 0)
    ], GlobalModule.prototype, "setUser", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], GlobalModule.prototype, "fetchUser", null);
    GlobalModule = tslib_1.__decorate([
        core_1.Module('Global')
    ], GlobalModule);
    return GlobalModule;
}(core_1.Ayanami));
var CountModule = /** @class */ (function (_super) {
    tslib_1.__extends(CountModule, _super);
    function CountModule(globalModule) {
        var _this = _super.call(this) || this;
        _this.globalModule = globalModule;
        _this.defaultState = {
            count: 0,
        };
        return _this;
    }
    CountModule.prototype.setCount = function (state, payload) {
        state.count = payload;
    };
    CountModule.prototype.addCount = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.withLatestFrom(this.state$), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.addCountAsync = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(ASYNC_DELAY_TIME), operators_1.withLatestFrom(this.state$), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.waitUserAddCount = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.combineLatest(this.globalModule.state$.pipe(operators_1.skipWhile(function (v) { return !v.user; }), operators_1.take(1)), this.state$.pipe(operators_1.take(1))), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 3), state = _b[2];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    tslib_1.__decorate([
        core_1.ImmerReducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Number]),
        tslib_1.__metadata("design:returntype", void 0)
    ], CountModule.prototype, "setCount", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], CountModule.prototype, "addCount", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], CountModule.prototype, "addCountAsync", null);
    tslib_1.__decorate([
        core_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", void 0)
    ], CountModule.prototype, "waitUserAddCount", null);
    CountModule = tslib_1.__decorate([
        core_1.Module('CountModule'),
        tslib_1.__metadata("design:paramtypes", [GlobalModule])
    ], CountModule);
    return CountModule;
}(core_1.Ayanami));
describe('AyanamiTestModule specs', function () {
    var globalStub;
    var countStub;
    var fakeTimer;
    beforeEach(function () {
        var testModule = di_1.Test.createTestingModule({
            TestModule: testbed_1.AyanamiTestModule,
            providers: [GlobalModule, CountModule],
        }).compile();
        globalStub = testModule.getAyanamiTestingStub(GlobalModule);
        countStub = testModule.getAyanamiTestingStub(CountModule);
        fakeTimer = sinon_1.useFakeTimers();
    });
    afterEach(function () {
        fakeTimer.restore();
    });
    it('should get state from global module', function () {
        var defaultState = globalStub.getState();
        expect(defaultState).toEqual({ user: '' });
    });
    it('should get state from count module', function () {
        var defaultState = countStub.getState();
        expect(defaultState).toEqual({ count: 0 });
    });
    it('should get dispatcher', function () {
        countStub.dispatcher.waitUserAddCount();
        globalStub.dispatcher.fetchUser();
        fakeTimer.tick(ASYNC_DELAY_TIME);
        expect(countStub.getState()).toEqual({ count: 1 });
    });
});
