"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("reflect-metadata");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var di_1 = require("@asuka/di");
var Sinon = tslib_1.__importStar(require("sinon"));
var rxjs_2 = require("rxjs");
var ayanami_1 = require("../ayanami");
var module_decorator_1 = require("../module.decorator");
var decorators_1 = require("../decorators");
var TIME_TO_DELAY = 300;
var Counter = /** @class */ (function (_super) {
    tslib_1.__extends(Counter, _super);
    function Counter() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = {
            count: 1,
        };
        return _this;
    }
    Counter.prototype.setCountImmer = function (state, payload) {
        state.count = payload.valueOf();
    };
    Counter.prototype.addCountOneImmer = function (state) {
        state.count = state.count + 1;
    };
    Counter.prototype.addOne = function (state) {
        return tslib_1.__assign(tslib_1.__assign({}, state), { count: state.count + 1 });
    };
    Counter.prototype.setCount = function (state, payload) {
        return tslib_1.__assign(tslib_1.__assign({}, state), { count: payload });
    };
    Counter.prototype.asyncAddCount = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(TIME_TO_DELAY), operators_1.map(function (payload) { return _this.getActions().asyncAddCountString("" + payload); }), operators_1.takeUntil(this.dispose$));
    };
    Counter.prototype.asyncAddCountString = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(TIME_TO_DELAY), operators_1.map(function (payload) { return _this.getActions().setCount(parseInt(payload)); }), operators_1.takeUntil(this.getAction$().asyncAddCount));
    };
    Counter.prototype.asyncAddCountOne = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.delay(TIME_TO_DELAY), operators_1.withLatestFrom(this.state$), operators_1.map(function (_a) {
            var _b = tslib_1.__read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    Counter.prototype.effectWithPureSideEffect = function (payload$) {
        var _this = this;
        return payload$.pipe(operators_1.tap(rxjs_2.noop), operators_1.map(function () { return _this.createNoopAction(); }));
    };
    tslib_1.__decorate([
        decorators_1.DefineAction(),
        tslib_1.__metadata("design:type", rxjs_1.Observable)
    ], Counter.prototype, "dispose$", void 0);
    tslib_1.__decorate([
        decorators_1.ImmerReducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Date]),
        tslib_1.__metadata("design:returntype", void 0)
    ], Counter.prototype, "setCountImmer", null);
    tslib_1.__decorate([
        decorators_1.ImmerReducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object]),
        tslib_1.__metadata("design:returntype", void 0)
    ], Counter.prototype, "addCountOneImmer", null);
    tslib_1.__decorate([
        decorators_1.Reducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object]),
        tslib_1.__metadata("design:returntype", void 0)
    ], Counter.prototype, "addOne", null);
    tslib_1.__decorate([
        decorators_1.Reducer(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Number]),
        tslib_1.__metadata("design:returntype", void 0)
    ], Counter.prototype, "setCount", null);
    tslib_1.__decorate([
        decorators_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], Counter.prototype, "asyncAddCount", null);
    tslib_1.__decorate([
        decorators_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], Counter.prototype, "asyncAddCountString", null);
    tslib_1.__decorate([
        decorators_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", rxjs_1.Observable)
    ], Counter.prototype, "asyncAddCountOne", null);
    tslib_1.__decorate([
        decorators_1.Effect(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
        tslib_1.__metadata("design:returntype", void 0)
    ], Counter.prototype, "effectWithPureSideEffect", null);
    Counter = tslib_1.__decorate([
        module_decorator_1.Module('counter')
    ], Counter);
    return Counter;
}(ayanami_1.Ayanami));
describe('Ayanami Class', function () {
    var counter;
    beforeEach(function () {
        counter = di_1.rootInjector.resolveAndInstantiate(Counter);
    });
    describe('basic shape specs', function () {
        it('should dispatch reducer action', function () {
            expect(counter.getActions().setCount(1)).toStrictEqual({
                payload: 1,
                type: 'setCount',
            });
        });
        it('should dispatch effect action', function () {
            expect(counter.getActions().asyncAddCount(1)).toStrictEqual({
                payload: 1,
                type: 'asyncAddCount',
            });
        });
        it('should dispatch immer reducer action', function () {
            var payload = new Date();
            expect(counter.getActions().setCountImmer(payload)).toStrictEqual({
                payload: payload,
                type: 'setCountImmer',
            });
        });
        it('should dispatch reducer action with void payload', function () {
            expect(counter.getActions().addOne()).toStrictEqual({
                payload: undefined,
                type: 'addOne',
            });
        });
        it('should dispatch effect action with void payload', function () {
            expect(counter.getActions().asyncAddCountOne()).toStrictEqual({
                payload: undefined,
                type: 'asyncAddCountOne',
            });
        });
        it('should dispatch immer reducer action with void payload', function () {
            expect(counter.getActions().addCountOneImmer()).toStrictEqual({
                payload: undefined,
                type: 'addCountOneImmer',
            });
        });
        it('should dispatch action created by DefineAction', function () {
            expect(counter.getActions().dispose$()).toStrictEqual({
                payload: undefined,
                type: 'dispose$',
            });
        });
    });
    describe('complex shape specs', function () {
        it('should be able to create module with without epic', function () {
            var WithoutEpic = /** @class */ (function (_super) {
                tslib_1.__extends(WithoutEpic, _super);
                function WithoutEpic() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {
                        count: 0,
                    };
                    return _this;
                }
                WithoutEpic.prototype.set = function (state, payload) {
                    return tslib_1.__assign(tslib_1.__assign({}, state), { count: payload });
                };
                tslib_1.__decorate([
                    decorators_1.Reducer(),
                    tslib_1.__metadata("design:type", Function),
                    tslib_1.__metadata("design:paramtypes", [Object, Number]),
                    tslib_1.__metadata("design:returntype", void 0)
                ], WithoutEpic.prototype, "set", null);
                WithoutEpic = tslib_1.__decorate([
                    module_decorator_1.Module('WithoutEpic')
                ], WithoutEpic);
                return WithoutEpic;
            }(ayanami_1.Ayanami));
            var onlyReducer = new WithoutEpic();
            var state = onlyReducer.createState();
            var actionCreator = onlyReducer.getActions();
            state.dispatch(actionCreator.set(1));
            expect(state.getState().count).toBe(1);
        });
        it('should be able to create module without reducer', function () {
            var WithoutReducer = /** @class */ (function (_super) {
                tslib_1.__extends(WithoutReducer, _super);
                function WithoutReducer() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {
                        count: 0,
                    };
                    return _this;
                }
                WithoutReducer.prototype.set = function (payload$) {
                    var _this = this;
                    return payload$.pipe(operators_1.tap(rxjs_2.noop), operators_1.map(function () { return _this.createNoopAction(); }));
                };
                tslib_1.__decorate([
                    decorators_1.Effect(),
                    tslib_1.__metadata("design:type", Function),
                    tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
                    tslib_1.__metadata("design:returntype", void 0)
                ], WithoutReducer.prototype, "set", null);
                WithoutReducer = tslib_1.__decorate([
                    module_decorator_1.Module('WithoutReducer')
                ], WithoutReducer);
                return WithoutReducer;
            }(ayanami_1.Ayanami));
            var withoutReducer = new WithoutReducer();
            var state = withoutReducer.createState();
            var actionCreator = withoutReducer.getActions();
            state.dispatch(actionCreator.set(1));
            expect(state.getState().count).toBe(withoutReducer.defaultState.count);
        });
        it('should throw if module name conflict#1', function () {
            var fn = function () {
                var Module1 = /** @class */ (function (_super) {
                    tslib_1.__extends(Module1, _super);
                    function Module1() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module1 = tslib_1.__decorate([
                        module_decorator_1.Module('Module')
                    ], Module1);
                    return Module1;
                }(ayanami_1.Ayanami));
                var Module2 = /** @class */ (function (_super) {
                    tslib_1.__extends(Module2, _super);
                    function Module2() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module2 = tslib_1.__decorate([
                        module_decorator_1.Module('Module')
                    ], Module2);
                    return Module2;
                }(ayanami_1.Ayanami));
                return { Module1: Module1, Module2: Module2 };
            };
            expect(fn).toThrow();
        });
        it('should throw if module name conflict#2', function () {
            var fn = function () {
                var Module1 = /** @class */ (function (_super) {
                    tslib_1.__extends(Module1, _super);
                    function Module1() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module1 = tslib_1.__decorate([
                        module_decorator_1.Module('Module1')
                    ], Module1);
                    return Module1;
                }(ayanami_1.Ayanami));
                var Module2 = /** @class */ (function (_super) {
                    tslib_1.__extends(Module2, _super);
                    function Module2() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module2 = tslib_1.__decorate([
                        module_decorator_1.Module('Module1')
                    ], Module2);
                    return Module2;
                }(ayanami_1.Ayanami));
                return { Module1: Module1, Module2: Module2 };
            };
            expect(fn).toThrow();
        });
        it('should throw if config in module invalid', function () {
            var fn = function () {
                var Module1 = /** @class */ (function (_super) {
                    tslib_1.__extends(Module1, _super);
                    function Module1() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module1 = tslib_1.__decorate([
                        module_decorator_1.Module({})
                    ], Module1);
                    return Module1;
                }(ayanami_1.Ayanami));
                return Module1;
            };
            expect(fn).toThrow();
        });
    });
    describe('dispatcher', function () {
        var state;
        var actionsDispatcher;
        var spy;
        var timer;
        beforeEach(function () {
            var actions = counter.getActions();
            state = counter.createState();
            actionsDispatcher = Object.keys(actions).reduce(function (acc, key) {
                acc[key] = function (p) {
                    var action = actions[key](p);
                    state.dispatch(action);
                    return action;
                };
                return acc;
            }, {});
            spy = Sinon.spy();
            state.subscribeAction(spy);
            timer = Sinon.useFakeTimers();
        });
        afterEach(function () {
            spy.resetHistory();
            timer.restore();
            state.unsubscribe();
        });
        it('should be able to dispatch reducer action by actions dispatcher #void', function () {
            var action = actionsDispatcher.addOne();
            var _a = tslib_1.__read(spy.args, 1), _b = tslib_1.__read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count + 1);
        });
        it('should be able to dispatch reducer action by actions dispatcher #param', function () {
            var newCount = 100;
            var action = actionsDispatcher.setCount(newCount);
            var _a = tslib_1.__read(spy.args, 1), _b = tslib_1.__read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(newCount);
        });
        it('should be able to dispatch immer reducer action by actions dispatcher #void', function () {
            var action = actionsDispatcher.addCountOneImmer();
            var _a = tslib_1.__read(spy.args, 1), _b = tslib_1.__read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count + 1);
        });
        it('should be able to dispatch immer reducer action by actions dispatcher #param', function () {
            var newCount = new Date();
            var action = actionsDispatcher.setCountImmer(newCount);
            var _a = tslib_1.__read(spy.args, 1), _b = tslib_1.__read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(newCount.valueOf());
        });
        it('should be able to dispatch epic action by actions dispatcher #void', function () {
            var action = actionsDispatcher.asyncAddCountOne();
            var _a = tslib_1.__read(spy.args, 1), _b = tslib_1.__read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count);
            timer.tick(TIME_TO_DELAY);
            var _c = tslib_1.__read(spy.args, 2), _d = tslib_1.__read(_c[1], 1), arg1 = _d[0];
            expect(arg1).toStrictEqual(counter.getActions().setCount(counter.defaultState.count + 1));
            expect(spy.callCount).toBe(2);
        });
        it('should be able to dispatch epic action by actions dispatcher #param', function () {
            var newCount = 100;
            var action = actionsDispatcher.asyncAddCount(newCount);
            var _a = tslib_1.__read(spy.args, 1), _b = tslib_1.__read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count);
            timer.tick(TIME_TO_DELAY);
            var _c = tslib_1.__read(spy.args, 2), _d = tslib_1.__read(_c[1], 1), arg1 = _d[0];
            expect(arg1).toStrictEqual(counter.getActions().asyncAddCountString("" + newCount));
            expect(spy.callCount).toBe(2);
        });
        it('should be able to dispose by other actions', function () {
            var newCount = 100;
            actionsDispatcher.asyncAddCount(newCount);
            expect(spy.callCount).toBe(1);
            actionsDispatcher.dispose$();
            timer.tick(TIME_TO_DELAY);
            expect(spy.callCount).toBe(2);
            expect(state.getState()).toStrictEqual(counter.defaultState);
        });
        it('should be able to dispatch noop action', function () {
            var action = actionsDispatcher.effectWithPureSideEffect();
            var _a = tslib_1.__read(spy.args, 2), _b = tslib_1.__read(_a[0], 1), arg1 = _b[0], _c = tslib_1.__read(_a[1], 1), arg2 = _c[0];
            expect(arg1).toStrictEqual(action);
            expect(typeof arg2.type).toBe('symbol');
            expect(spy.callCount).toBe(2);
            expect(state.getState()).toStrictEqual(counter.defaultState);
        });
    });
});
