"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("reflect-metadata");
var di_1 = require("@asuka/di");
var Sinon = tslib_1.__importStar(require("sinon"));
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var state_1 = require("../state");
var ayanami_1 = require("../ayanami");
var decorators_1 = require("../decorators");
var module_decorator_1 = require("../module.decorator");
describe('Smoking tests', function () {
    it('Module should be able to work without effects', function () {
        var stateCreator = state_1.createState(function (state, action) {
            if (action.type === 'foo') {
                return tslib_1.__assign(tslib_1.__assign({}, state), { name: action.payload });
            }
            return state;
        }, function () { return rxjs_1.empty(); }).stateCreator;
        var state = stateCreator({ name: 'bar' });
        var action = {
            type: 'foo',
            payload: 'foo',
            state: state,
        };
        state.dispatch(action);
        expect(state.getState().name).toBe(action.payload);
    });
    it('should be able to dispatch actions from the other module', function () {
        var asyncTimeToDelay = 2000;
        var timer = Sinon.useFakeTimers();
        var FooModule = /** @class */ (function (_super) {
            tslib_1.__extends(FooModule, _super);
            function FooModule() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {
                    foo: '1',
                };
                return _this;
            }
            FooModule.prototype.set = function (state, payload) {
                return tslib_1.__assign(tslib_1.__assign({}, state), { foo: payload });
            };
            tslib_1.__decorate([
                decorators_1.Reducer(),
                tslib_1.__metadata("design:type", Function),
                tslib_1.__metadata("design:paramtypes", [Object, String]),
                tslib_1.__metadata("design:returntype", void 0)
            ], FooModule.prototype, "set", null);
            FooModule = tslib_1.__decorate([
                module_decorator_1.Module('Foo')
            ], FooModule);
            return FooModule;
        }(ayanami_1.Ayanami));
        var BarModule = /** @class */ (function (_super) {
            tslib_1.__extends(BarModule, _super);
            function BarModule(fooModule) {
                var _this = _super.call(this) || this;
                _this.fooModule = fooModule;
                _this.defaultState = {};
                return _this;
            }
            BarModule.prototype.asyncSetFoo = function (payload$) {
                var _this = this;
                return payload$.pipe(operators_1.delay(asyncTimeToDelay), operators_1.map(function (payload) { return _this.fooModule.getActions().set(payload); }));
            };
            tslib_1.__decorate([
                decorators_1.Effect(),
                tslib_1.__metadata("design:type", Function),
                tslib_1.__metadata("design:paramtypes", [rxjs_1.Observable]),
                tslib_1.__metadata("design:returntype", void 0)
            ], BarModule.prototype, "asyncSetFoo", null);
            BarModule = tslib_1.__decorate([
                module_decorator_1.Module('Bar'),
                tslib_1.__metadata("design:paramtypes", [FooModule])
            ], BarModule);
            return BarModule;
        }(ayanami_1.Ayanami));
        var fooModuleState = di_1.rootInjector.getInstance(FooModule);
        var barModuleState = di_1.rootInjector.getInstance(BarModule);
        var fooState = fooModuleState.createState();
        var barState = barModuleState.createState();
        var payload = 'whatever';
        var action = barModuleState.getActions().asyncSetFoo(payload);
        expect(action.payload).toBe(payload);
        barState.dispatch(action);
        timer.tick(asyncTimeToDelay);
        expect(fooState.getState().foo).toBe(payload);
        timer.restore();
        di_1.rootInjector.reset();
    });
});
