"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var state_1 = require("../state");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var Sinon = tslib_1.__importStar(require("sinon"));
describe('state specs', function () {
    var UPDATE_FOO = 'update-foo';
    var UPDATE_BAR = 'update-bar';
    var ASYNC_UPDATE_FOO = 'async-update-foo';
    var ASYNC_UPDATE_BAR = 'async-update-bar';
    var mockReducer = function (prevState, action) {
        if (action.type === UPDATE_FOO) {
            return tslib_1.__assign(tslib_1.__assign({}, prevState), { foo: action.payload });
        }
        else if (action.type === UPDATE_BAR) {
            return tslib_1.__assign(tslib_1.__assign({}, prevState), { bar: action.payload });
        }
        return prevState;
    };
    var state;
    var mockEpic = function (action$) {
        return action$.pipe(operators_1.filter(function (_a) {
            var type = _a.type;
            return type === ASYNC_UPDATE_FOO;
        }), operators_1.map(function () { return ({ type: 'noop', payload: null, state: state }); }));
    };
    describe('stateCreator', function () {
        var defaultState = {
            foo: '1',
            bar: null,
        };
        var stateCreator = state_1.createState(mockReducer, rxjs_1.identity).stateCreator;
        it('should be able to create state', function () {
            expect(stateCreator(defaultState).getState()).toBe(defaultState);
        });
        it('should be able to create state with new defaultState', function () {
            var newDefaultState = tslib_1.__assign({}, defaultState);
            expect(stateCreator(newDefaultState).getState()).toBe(newDefaultState);
        });
    });
    describe('state', function () {
        var defaultState = {
            foo: '1',
            bar: null,
        };
        it('should be able to change state via dispatch action', function () {
            var stateCreator = state_1.createState(mockReducer, mockEpic).stateCreator;
            var state = stateCreator(defaultState);
            state.dispatch({ type: UPDATE_FOO, payload: '2', state: state });
            expect(state.getState().foo).toBe('2');
        });
        it('should do nothing when action type is not matched any reducer', function () {
            var stateCreator = state_1.createState(mockReducer, mockEpic).stateCreator;
            var state = stateCreator(defaultState);
            state.dispatch({ type: '__NOOP__', payload: '2', state: state });
            expect(state.getState().foo).toBe(defaultState.foo);
        });
        it('should run epic', function () {
            var stateCreator = state_1.createState(mockReducer, mockEpic).stateCreator;
            var state = stateCreator(defaultState);
            state.dispatch({ type: UPDATE_FOO, payload: '2', state: state });
            expect(state.getState().foo).toBe('2');
        });
        describe('effect', function () {
            var stateCreator;
            var timer;
            var bar = 100;
            var delayTime = 300;
            beforeEach(function () {
                var _stateCreator = state_1.createState(mockReducer, function (action$) {
                    return action$.pipe(operators_1.filter(function (_a) {
                        var type = _a.type;
                        return type === ASYNC_UPDATE_BAR;
                    }), operators_1.delay(delayTime), operators_1.map(function () { return ({
                        type: UPDATE_BAR,
                        payload: bar,
                        state: state,
                    }); }));
                }).stateCreator;
                stateCreator = _stateCreator;
                timer = Sinon.useFakeTimers();
            });
            afterEach(function () {
                timer.restore();
            });
            it('should be able to change state by effect', function () {
                state = stateCreator(defaultState);
                state.dispatch({
                    type: ASYNC_UPDATE_BAR,
                    payload: null,
                    state: state,
                });
                timer.tick(delayTime);
                expect(state.getState().bar).toBe(bar);
            });
        });
    });
});
