"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var immer_1 = tslib_1.__importDefault(require("immer"));
var state_1 = require("./state");
var symbols_1 = require("./symbols");
var constants_1 = require("../ssr/constants");
var redux_devtools_extension_1 = require("../redux-devtools-extension");
var _globalThis = 
/* istanbul ignore next */ typeof globalThis === 'undefined'
    ? /* istanbul ignore next */ typeof window === 'undefined'
        ? global
        : window
    : globalThis;
// cast to string to match the shape of action
var NOOP_ACTION_TYPE = Symbol('NOOP_ACTION');
var effectNameSymbol = Symbol('effect-name');
var Ayanami = /** @class */ (function () {
    function Ayanami() {
        var _this = this;
        // @internal
        this._actionKeys = [];
        // @internal
        this.state = null;
        this.effect = this.combineEffects();
        this.reducer = this.combineReducers();
        var _a = state_1.createState(this.reducer, this.effect), stateCreator = _a.stateCreator, action$ = _a.action$, state$ = _a.state$;
        this.stateCreator = stateCreator;
        this.action$ = action$;
        this.state$ = state$;
        this.combineDefineActions();
        this._actions = this._actionKeys.reduce(function (acc, key) {
            acc[key] = function (payload) {
                var action = {
                    type: key,
                    payload: payload,
                };
                Object.defineProperty(action, 'state', {
                    value: _this.state,
                    enumerable: false,
                    configurable: false,
                    writable: false,
                });
                return action;
            };
            return acc;
        }, Object.create(null));
        this._actionStreams = this._actionKeys.reduce(function (acc, key) {
            acc[key] = _this.action$.pipe(operators_1.filter(function (_a) {
                var type = _a.type;
                return type === key;
            }), operators_1.map(function (_a) {
                var payload = _a.payload;
                return payload;
            }));
            return acc;
        }, {});
    }
    Ayanami.prototype.createState = function (middleware) {
        if (middleware === void 0) { middleware = rxjs_1.identity; }
        if (this.state)
            return this.state;
        var ssrCache = _globalThis[Symbol.for(Symbol.keyFor(constants_1.GLOBAL_KEY))];
        var loadFromSSR = false;
        var preloadState;
        if (ssrCache && ssrCache[this.moduleName]) {
            preloadState = ssrCache[this.moduleName];
            loadFromSSR = true;
        }
        this.state = this.stateCreator((preloadState !== null && preloadState !== void 0 ? preloadState : this.defaultState), middleware, loadFromSSR);
        Reflect.defineMetadata(constants_1.SSR_LOADED_KEY, loadFromSSR, this.state);
        if (process.env.NODE_ENV !== 'production') {
            Object.defineProperty(this.state, 'name', {
                value: this.moduleName,
                configurable: false,
                enumerable: false,
                writable: false,
            });
            redux_devtools_extension_1.logStateAction({
                type: redux_devtools_extension_1.INIT_ACTION_TYPE,
                state: this.state,
                payload: null,
            });
        }
        return this.state;
    };
    Ayanami.prototype.getActions = function () {
        return this._actions;
    };
    Ayanami.prototype.getAction$ = function () {
        return this._actionStreams;
    };
    Ayanami.prototype.createNoopAction = function () {
        return {
            type: NOOP_ACTION_TYPE,
            payload: null,
            state: this.state,
        };
    };
    Ayanami.prototype.combineEffects = function () {
        var _a;
        var _this = this;
        var effectKeys = Reflect.getMetadata(symbols_1.EFFECT_DECORATOR_SYMBOL, this.constructor) || [];
        (_a = this._actionKeys).push.apply(_a, tslib_1.__spread(effectKeys));
        var effects = effectKeys.map(function (property) {
            var effect = _this[property].bind(_this);
            Object.defineProperty(effect, effectNameSymbol, {
                value: property,
                enumerable: false,
                configurable: false,
                writable: false,
            });
            return effect;
        });
        return function (action$, loadFromSSR) {
            var actionsToSkip = Reflect.getMetadata(constants_1.ACTION_TO_SKIP_KEY, _this.constructor.prototype);
            return rxjs_1.merge.apply(void 0, tslib_1.__spread(effects.map(function (effect) {
                var effectActionType = effect[effectNameSymbol];
                var skipCount = loadFromSSR && actionsToSkip && actionsToSkip.has(effectActionType) ? 1 : 0;
                var payload$ = action$.pipe(operators_1.filter(function (_a) {
                    var type = _a.type;
                    return type === effectActionType;
                }), operators_1.skip(skipCount), operators_1.map(function (_a) {
                    var payload = _a.payload;
                    return payload;
                }));
                return effect(payload$);
            })));
        };
    };
    Ayanami.prototype.combineReducers = function () {
        var _a;
        var _this = this;
        var reducerKeys = Reflect.getMetadata(symbols_1.REDUCER_DECORATOR_SYMBOL, this.constructor) || [];
        var reducers = reducerKeys.reduce(function (acc, property) {
            acc[property] = _this[property].bind(_this);
            return acc;
        }, {});
        var immerReducerKeys = Reflect.getMetadata(symbols_1.IMMER_REDUCER_DECORATOR_SYMBOL, this.constructor) || [];
        (_a = this._actionKeys).push.apply(_a, tslib_1.__spread(reducerKeys, immerReducerKeys));
        var immerReducers = immerReducerKeys.reduce(function (acc, property) {
            acc[property] = _this[property].bind(_this);
            return acc;
        }, {});
        return function (prevState, action) {
            var type = action.type;
            if (typeof type === 'string') {
                if (reducers[type]) {
                    return reducers[type](prevState, action.payload);
                }
                else if (immerReducers[type]) {
                    return immer_1.default(prevState, function (draft) { return immerReducers[type](draft, action.payload); });
                }
            }
            return prevState;
        };
    };
    Ayanami.prototype.combineDefineActions = function () {
        var _a, e_1, _b;
        this._defineActionKeys =
            Reflect.getMetadata(symbols_1.DEFINE_ACTION_DECORATOR_SYMBOL, this.constructor) || [];
        (_a = this._actionKeys).push.apply(_a, tslib_1.__spread(this._defineActionKeys));
        var _loop_1 = function (actionType) {
            ;
            this_1[actionType] = this_1.action$.pipe(operators_1.filter(function (_a) {
                var type = _a.type;
                return type === actionType;
            }), operators_1.map(function (_a) {
                var payload = _a.payload;
                return payload;
            }), operators_1.publish(), operators_1.refCount());
        };
        var this_1 = this;
        try {
            for (var _c = tslib_1.__values(this._defineActionKeys), _d = _c.next(); !_d.done; _d = _c.next()) {
                var actionType = _d.value;
                _loop_1(actionType);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_b = _c.return)) _b.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    return Ayanami;
}());
exports.Ayanami = Ayanami;
