"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importStar(require("react"));
var di_1 = require("@asuka/di");
var immer_1 = tslib_1.__importDefault(require("immer"));
var operators_1 = require("rxjs/operators");
var ssr_context_1 = require("./ssr/ssr-context");
var ssr_states_1 = require("./ssr/ssr-states");
var ssr_oneshot_cache_1 = require("./ssr/ssr-oneshot-cache");
var constants_1 = require("./ssr/constants");
function _useAyanamiDispatchers(ayanami) {
    var appDispatcher = react_1.default.useMemo(function () {
        var state = ayanami.state;
        var actionsCreator = ayanami.getActions();
        return Object.keys(actionsCreator).reduce(function (acc, cur) {
            acc[cur] = function (payload) {
                var action = actionsCreator[cur](payload);
                state.dispatch(action);
            };
            return acc;
        }, Object.create(null));
    }, [ayanami]);
    return appDispatcher;
}
function useAyanamiDispatchers(A) {
    var ayanami = _useState(A).ayanami;
    return _useAyanamiDispatchers(ayanami);
}
exports.useAyanamiDispatchers = useAyanamiDispatchers;
function _useAyanamiState(state, selector, mutateStateOnFirstRendering) {
    var _a = tslib_1.__read(react_1.default.useState(function () {
        var initialState = state.getState();
        if (typeof mutateStateOnFirstRendering === 'function') {
            initialState = immer_1.default(initialState, mutateStateOnFirstRendering);
            state.state$.next(initialState);
        }
        return selector && !Reflect.getMetadata(constants_1.SSR_LOADED_KEY, state)
            ? selector(initialState)
            : initialState;
    }), 2), appState = _a[0], setState = _a[1];
    // https://reactjs.org/docs/hooks-faq.html#how-do-i-implement-getderivedstatefromprops
    // do not put subscribe in useEffect
    var subscription = react_1.default.useMemo(function () {
        return state.state$
            .pipe(operators_1.skip(1), operators_1.map(function (s) {
            if (Reflect.getMetadata(constants_1.SSR_LOADED_KEY, state)) {
                Reflect.deleteMetadata(constants_1.SSR_LOADED_KEY, state);
                return s;
            }
            else {
                return selector ? selector(s) : s;
            }
        }), operators_1.distinctUntilChanged())
            .subscribe(setState);
    }, [state]);
    react_1.default.useEffect(function () { return function () { return subscription.unsubscribe(); }; }, [state]);
    return appState;
}
function useAyanamiState(A, config) {
    var _a, _b;
    var state = _useState(A).state;
    return _useAyanamiState(state, (_a = config) === null || _a === void 0 ? void 0 : _a.selector, (_b = config) === null || _b === void 0 ? void 0 : _b.mutateStateOnFirstRendering);
}
exports.useAyanamiState = useAyanamiState;
function useAyanami(A, config) {
    var _a, _b;
    var _c = _useState(A), ayanami = _c.ayanami, state = _c.state;
    var appState = _useAyanamiState(state, (_a = config) === null || _a === void 0 ? void 0 : _a.selector, (_b = config) === null || _b === void 0 ? void 0 : _b.mutateStateOnFirstRendering);
    var appDispatcher = _useAyanamiDispatchers(ayanami);
    return [appState, appDispatcher];
}
exports.useAyanami = useAyanami;
function _useState(A) {
    var ssrSharedContext = react_1.useContext(ssr_context_1.SSRSharedContext);
    var ssrContext = react_1.useContext(ssr_context_1.SSRContext);
    var ayanami = di_1.useInstance(A);
    var state = react_1.default.useMemo(function () {
        var _a;
        return ssr_states_1.SSRStateCacheInstance.has(ssrSharedContext, A)
            ? ssr_states_1.SSRStateCacheInstance.get(ssrSharedContext, A)
            : ssrContext
                ? (_a = ssr_oneshot_cache_1.oneShotCache.consume(ssrContext, A), (_a !== null && _a !== void 0 ? _a : ayanami.createState())) : ayanami.createState();
    }, [ayanami, ssrContext, ssrSharedContext]);
    return { ayanami: ayanami, state: state };
}
