"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var sinon_1 = require("sinon");
var ssr_oneshot_cache_1 = require("../ssr-oneshot-cache");
var symbols_1 = require("../../core/symbols");
var unsubscribeSpy = sinon_1.spy();
var Normal = /** @class */ (function () {
    function Normal() {
    }
    return Normal;
}());
var MockAyanami = /** @class */ (function () {
    function MockAyanami() {
        var _a;
        this.state = (_a = {},
            _a[symbols_1.StateInterface] = {
                unsubscribe: unsubscribeSpy,
            },
            _a);
    }
    return MockAyanami;
}());
var MockAyanamiLevel2 = /** @class */ (function () {
    function MockAyanamiLevel2() {
        var _a;
        this.state = (_a = {},
            _a[symbols_1.StateInterface] = {
                unsubscribe: unsubscribeSpy,
            },
            _a);
    }
    return MockAyanamiLevel2;
}());
describe('SSROneShotCache specs', function () {
    var cache;
    var ctx = Symbol();
    beforeEach(function () {
        cache = new ssr_oneshot_cache_1.SSROneShotCache();
    });
    afterEach(function () {
        unsubscribeSpy.resetHistory();
    });
    it('should work with ayanami#1', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var instance;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    instance = new MockAyanami();
                    cache.store(ctx, MockAyanami, instance.state);
                    expect(cache.consume(ctx, MockAyanami)).toBe(instance.state);
                    return [4 /*yield*/, Promise.resolve()];
                case 1:
                    _a.sent();
                    expect(cache.consume(ctx, MockAyanami)).toBe(undefined);
                    expect(unsubscribeSpy.callCount).toBe(1);
                    return [2 /*return*/];
            }
        });
    }); });
    it('should work with ayanami#2', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var instance, instanceLevel2, normalInstance;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    instance = new MockAyanami();
                    instanceLevel2 = new MockAyanamiLevel2();
                    normalInstance = new Normal();
                    cache.store(ctx, MockAyanami, instance.state);
                    cache.store(ctx, MockAyanamiLevel2, instanceLevel2.state);
                    cache.store(ctx, Normal, normalInstance);
                    expect(cache.consume(ctx, MockAyanami)).toBe(instance.state);
                    expect(cache.consume(ctx, MockAyanamiLevel2)).toBe(instanceLevel2.state);
                    expect(cache.consume(ctx, Normal)).toBe(normalInstance);
                    return [4 /*yield*/, Promise.resolve()];
                case 1:
                    _a.sent();
                    expect(cache.consume(ctx, MockAyanami)).toBe(undefined);
                    expect(cache.consume(ctx, MockAyanamiLevel2)).toBe(undefined);
                    expect(cache.consume(ctx, Normal)).toBe(undefined);
                    expect(unsubscribeSpy.callCount).toBe(2);
                    return [2 /*return*/];
            }
        });
    }); });
    it('should work with non-ayanami instance', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var instance;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    instance = new Normal();
                    cache.store(ctx, Normal, instance);
                    expect(cache.consume(ctx, Normal)).toBe(instance);
                    return [4 /*yield*/, Promise.resolve()];
                case 1:
                    _a.sent();
                    expect(cache.consume(ctx, Normal)).toBe(undefined);
                    expect(unsubscribeSpy.callCount).toBe(0);
                    return [2 /*return*/];
            }
        });
    }); });
});
