"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var sinon_1 = tslib_1.__importDefault(require("sinon"));
var rxjs_1 = require("rxjs");
var ssr_states_1 = require("../ssr-states");
var core_1 = require("../../core");
describe('SSRStateCacheInstance specs:', function () {
    var SSRStateCacheInstance;
    var A = /** @class */ (function (_super) {
        tslib_1.__extends(A, _super);
        function A() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.defaultState = {};
            return _this;
        }
        return A;
    }(core_1.Ayanami));
    var ctx = {};
    var state = Object.create({
        unsubscribe: rxjs_1.noop,
    });
    beforeEach(function () {
        SSRStateCacheInstance = new ssr_states_1.SSRStateCache();
        SSRStateCacheInstance.set(ctx, A, state);
    });
    it('has should work', function () {
        expect(SSRStateCacheInstance.has(ctx, A)).toBe(true);
    });
    it('get existed should work', function () {
        expect(SSRStateCacheInstance.get(ctx, A)).toBe(state);
    });
    it('get not existed should return undefined', function () {
        expect(SSRStateCacheInstance.get({}, A)).toBe(undefined);
    });
    it('should clear all if pool size smaller than cache size', function () {
        SSRStateCacheInstance.setPoolSize(1);
        expect(SSRStateCacheInstance.get({}, A)).toBe(undefined);
        var newCtx = {};
        var newState = Object.create(null);
        SSRStateCacheInstance.set(newCtx, A, newState);
        expect(SSRStateCacheInstance.get(newCtx, A)).toBe(newState);
        SSRStateCacheInstance.set(ctx, A, state);
        expect(SSRStateCacheInstance.get(newCtx, A)).toBe(undefined);
    });
    it('should delete all states after cleanupStateByContext', function () {
        var spy = sinon_1.default.spy();
        var newState = Object.create({
            unsubscribe: spy,
        });
        var B = /** @class */ (function (_super) {
            tslib_1.__extends(B, _super);
            function B() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {};
                return _this;
            }
            return B;
        }(core_1.Ayanami));
        SSRStateCacheInstance.set(ctx, B, newState);
        expect(SSRStateCacheInstance.get(ctx, B)).toBe(newState);
        SSRStateCacheInstance.cleanupStateByContext(ctx);
        expect(SSRStateCacheInstance.get(ctx, A)).toBeUndefined();
        expect(SSRStateCacheInstance.get(ctx, B)).toBeUndefined();
        expect(spy.callCount).toBe(1);
    });
    it('should do nothing when cleanupStateByContext non-existed ctx', function () {
        SSRStateCacheInstance.cleanupStateByContext({});
        expect(SSRStateCacheInstance.get(ctx, A)).toBe(state);
    });
});
