"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var di_1 = require("@asuka/di");
var constants_1 = require("./constants");
var ssr_effect_1 = require("./ssr-effect");
var terminate_1 = require("./terminate");
var ssr_states_1 = require("./ssr-states");
var ssr_oneshot_cache_1 = require("./ssr-oneshot-cache");
var skipFn = function () { return ssr_effect_1.SKIP_SYMBOL; };
/**
 * Run all @SSREffect decorated effects of given modules and extract latest states.
 * `cleanup` function returned must be called before end of responding
 *
 * @param ctx request context, which will be passed to payloadGetter in SSREffect decorator param
 * @param modules used ayanami modules
 * @param uuid the same uuid would reuse the same state which was created before
 * @param timeout seconds to wait before all effects stream out TERMINATE_ACTION
 * @returns object contains ayanami state and cleanup function
 */
exports.runSSREffects = function (ctx, modules, sharedCtx, timeout) {
    if (timeout === void 0) { timeout = 3; }
    var stateToSerialize = {};
    return modules.length === 0
        ? Promise.resolve(stateToSerialize)
        : rxjs_1.race(rxjs_1.from(modules).pipe(operators_1.flatMap(function (constructor) {
            return new rxjs_1.Observable(function (observer) {
                var cleanup = rxjs_1.noop;
                var metas = Reflect.getMetadata(constants_1.SSRSymbol, constructor.prototype) || [];
                var ayanamiState;
                var moduleName;
                var middleware = function (effect$) {
                    return effect$.pipe(operators_1.tap({
                        error: function (e) {
                            observer.error(e);
                        },
                    }));
                };
                if (sharedCtx) {
                    if (ssr_states_1.SSRStateCacheInstance.has(sharedCtx, constructor)) {
                        ayanamiState = ssr_states_1.SSRStateCacheInstance.get(sharedCtx, constructor);
                        moduleName = constructor.prototype.moduleName;
                    }
                    else {
                        var ayanamiInstance = di_1.rootInjector.resolveAndInstantiate(constructor);
                        moduleName = ayanamiInstance.moduleName;
                        ayanamiState = ayanamiInstance.createState(middleware);
                        ssr_states_1.SSRStateCacheInstance.set(sharedCtx, constructor, ayanamiState);
                    }
                }
                else {
                    var ayanamiInstance = di_1.rootInjector.resolveAndInstantiate(constructor);
                    moduleName = ayanamiInstance.moduleName;
                    ayanamiState = ayanamiInstance.createState(middleware);
                    ssr_oneshot_cache_1.oneShotCache.store(ctx, constructor, ayanamiState);
                }
                var effectsCount = metas.length;
                var disposeFn = rxjs_1.noop;
                cleanup = sharedCtx
                    ? function () { return disposeFn(); }
                    : function () {
                        ayanamiState.unsubscribe();
                    };
                function runEffects() {
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        var action$_1, state;
                        var _this = this;
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, Promise.all(metas.map(function (meta) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                        var param;
                                        return tslib_1.__generator(this, function (_a) {
                                            switch (_a.label) {
                                                case 0:
                                                    if (!meta.middleware) return [3 /*break*/, 2];
                                                    return [4 /*yield*/, meta.middleware(ctx, skipFn)];
                                                case 1:
                                                    param = _a.sent();
                                                    if (param !== ssr_effect_1.SKIP_SYMBOL) {
                                                        ayanamiState.dispatch({
                                                            type: meta.action,
                                                            payload: param,
                                                            state: ayanamiState,
                                                        });
                                                    }
                                                    else {
                                                        effectsCount -= 1;
                                                    }
                                                    return [3 /*break*/, 3];
                                                case 2:
                                                    ayanamiState.dispatch({
                                                        type: meta.action,
                                                        payload: undefined,
                                                        state: ayanamiState,
                                                    });
                                                    _a.label = 3;
                                                case 3: return [2 /*return*/];
                                            }
                                        });
                                    }); }))];
                                case 1:
                                    _a.sent();
                                    if (!(effectsCount > 0)) return [3 /*break*/, 3];
                                    action$_1 = new rxjs_1.Subject();
                                    disposeFn = ayanamiState.subscribeAction(function (action) {
                                        action$_1.next(action);
                                    });
                                    return [4 /*yield*/, action$_1
                                            .pipe(operators_1.filter(function (act) { return act.type === terminate_1.TERMINATE_ACTION.type; }), operators_1.bufferCount(effectsCount), operators_1.take(1))
                                            .toPromise()];
                                case 2:
                                    _a.sent();
                                    state = ayanamiState.getState();
                                    stateToSerialize[moduleName] = state;
                                    _a.label = 3;
                                case 3: return [2 /*return*/];
                            }
                        });
                    });
                }
                runEffects()
                    .then(function () {
                    observer.next(stateToSerialize);
                    observer.complete();
                })
                    .catch(function (e) {
                    observer.error(e);
                });
                return cleanup;
            });
        })), rxjs_1.timer(timeout * 1000).pipe(operators_1.flatMap(function () { return rxjs_1.throwError(new Error('Terminate timeout')); }))).toPromise();
};
