export declare const SKIP_SYMBOL: unique symbol;
interface SSREffectOptions<Context, Payload> {
    /**
     * Function used to get effect payload.
     *
     * if SKIP_SYMBOL returned(`return skip()`), effect won't get dispatched when SSR
     *
     * @param req express request object
     * @param skip get a symbol used to let effect escape from ssr effects dispatching
     */
    payloadGetter?: (req: Context, skip: () => typeof SKIP_SYMBOL) => Payload | Promise<Payload> | typeof SKIP_SYMBOL;
    /**
     * Whether skip first effect dispatching in client if effect ever got dispatched when SSR
     *
     * @default true
     */
    skipFirstClientDispatch?: boolean;
}
export declare function SSREffect<T, Context, Payload>(options?: SSREffectOptions<Context, Payload>): (target: T, method: string, descriptor: PropertyDescriptor) => PropertyDescriptor;
export {};
