"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var constants_1 = require("./constants");
var decorators_1 = require("../core/decorators");
exports.SKIP_SYMBOL = Symbol('skip');
function addDecorator(target, method, middleware) {
    var existedMetas = Reflect.getMetadata(constants_1.SSRSymbol, target);
    var meta = { action: method, middleware: middleware };
    if (existedMetas) {
        existedMetas.push(meta);
    }
    else {
        Reflect.defineMetadata(constants_1.SSRSymbol, [meta], target);
    }
}
function SSREffect(options) {
    var _a = tslib_1.__assign({ payloadGetter: undefined, skipFirstClientDispatch: true }, options), payloadGetter = _a.payloadGetter, skipFirstClientDispatch = _a.skipFirstClientDispatch;
    return function (target, method, descriptor) {
        addDecorator(target, method, payloadGetter);
        if (skipFirstClientDispatch) {
            var actionsToSkip = Reflect.getMetadata(constants_1.ACTION_TO_SKIP_KEY, target) || new Set();
            actionsToSkip.add(method);
            Reflect.defineMetadata(constants_1.ACTION_TO_SKIP_KEY, actionsToSkip, target);
        }
        return decorators_1.Effect()(target, method, descriptor);
    };
}
exports.SSREffect = SSREffect;
