import { __decorate, __extends, __metadata, __read } from "tslib";
import 'reflect-metadata';
import React, { useEffect, useCallback } from 'react';
import { useAyanami, useAyanamiState, useAyanamiDispatchers } from '../hooks';
import { Ayanami, Effect, ImmerReducer, Module, Reducer } from '../core';
import { Observable } from 'rxjs';
import { map, withLatestFrom, delay, skipWhile, combineLatest, take } from 'rxjs/operators';
import { act, create } from 'react-test-renderer';
import { rootInjector, InjectableContext } from '@asuka/di';
import Sinon from 'sinon';
var ASYNC_DELAY_TIME = 1000;
var GlobalModule = /** @class */ (function (_super) {
    __extends(GlobalModule, _super);
    function GlobalModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = {
            user: '',
        };
        return _this;
    }
    GlobalModule.prototype.setUser = function (state, payload) {
        state.user = payload;
    };
    GlobalModule.prototype.fetchUser = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(ASYNC_DELAY_TIME), map(function () { return _this.getActions().setUser('global'); }));
    };
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, String]),
        __metadata("design:returntype", void 0)
    ], GlobalModule.prototype, "setUser", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], GlobalModule.prototype, "fetchUser", null);
    GlobalModule = __decorate([
        Module('Global')
    ], GlobalModule);
    return GlobalModule;
}(Ayanami));
var CountModule = /** @class */ (function (_super) {
    __extends(CountModule, _super);
    function CountModule(globalModule) {
        var _this = _super.call(this) || this;
        _this.globalModule = globalModule;
        _this.defaultState = {
            count: 0,
        };
        return _this;
    }
    CountModule.prototype.setCount = function (state, payload) {
        state.count = payload;
    };
    CountModule.prototype.addCount = function (payload$) {
        var _this = this;
        return payload$.pipe(withLatestFrom(this.state$), map(function (_a) {
            var _b = __read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.addCountAsync = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(ASYNC_DELAY_TIME), withLatestFrom(this.state$), map(function (_a) {
            var _b = __read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.waitUserAddCount = function (payload$) {
        var _this = this;
        return payload$.pipe(combineLatest(this.globalModule.state$.pipe(skipWhile(function (v) { return !v.user; }), take(1)), this.state$.pipe(take(1))), map(function (_a) {
            var _b = __read(_a, 3), state = _b[2];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Number]),
        __metadata("design:returntype", void 0)
    ], CountModule.prototype, "setCount", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModule.prototype, "addCount", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModule.prototype, "addCountAsync", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModule.prototype, "waitUserAddCount", null);
    CountModule = __decorate([
        Module('CountModule'),
        __metadata("design:paramtypes", [GlobalModule])
    ], CountModule);
    return CountModule;
}(Ayanami));
describe('Hooks specs: useAyanami', function () {
    var SimpleComponent = function () {
        var _a = __read(useAyanami(CountModule), 2), state = _a[0], actions = _a[1];
        var addCount = useCallback(function () {
            actions.addCount();
        }, []);
        var addCountAsync = useCallback(function () {
            actions.addCountAsync();
        }, []);
        return (React.createElement("div", null,
            React.createElement("span", null, state.count),
            React.createElement("button", { id: "add-count", onClick: addCount }, "+"),
            React.createElement("button", { id: "add-count-async", onClick: addCountAsync }, "+"),
            React.createElement("button", { id: "set-count", onClick: function () { return actions.setCount(10); } }, "set")));
    };
    var testRenderer;
    var getCount;
    var timer;
    beforeEach(function () {
        timer = Sinon.useFakeTimers();
        testRenderer = create(React.createElement(InjectableContext, null,
            React.createElement(SimpleComponent, null)));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        timer.restore();
        rootInjector.reset();
        rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should render correct state', function () {
        expect(getCount()).toBe(0);
    });
    it('should dispatch action', function () {
        act(function () {
            testRenderer.root.find(function (instance) { return instance.props.id === 'set-count'; }).props.onClick();
        });
        expect(getCount()).toBe(10);
    });
    it('should dispatch effect', function () {
        expect(getCount()).toBe(0);
        act(function () {
            testRenderer.root.find(function (instance) { return instance.props.id === 'add-count'; }).props.onClick();
        });
        expect(getCount()).toBe(1);
    });
    it('should dispatch async effect', function () {
        act(function () {
            testRenderer.root.find(function (instance) { return instance.props.id === 'add-count-async'; }).props.onClick();
            timer.tick(ASYNC_DELAY_TIME);
        });
        expect(getCount()).toBe(1);
    });
    it('should only render once when update state right during rendering', function () {
        var spyFn = jest.fn();
        var TestComponent = function () {
            var _a = __read(useAyanami(CountModule), 2), state = _a[0], actions = _a[1];
            if (state.count === 2) {
                actions.addCount();
            }
            useEffect(function () {
                spyFn(state.count);
            }, [state.count]);
            return (React.createElement("div", null,
                React.createElement("span", null, state.count),
                React.createElement("button", { id: "set-count", onClick: function () { return actions.setCount(2); } }, "set")));
        };
        var renderer = create(React.createElement(TestComponent, null));
        // https://github.com/facebook/react/issues/14050 to trigger useEffect manually
        renderer.update(React.createElement(TestComponent, null));
        expect(spyFn.mock.calls).toEqual([[0]]);
        act(function () {
            renderer.root.findByType('button').props.onClick();
        });
        expect(spyFn.mock.calls).toEqual([[0], [3]]);
    });
    it('should be able to use state$ from the other module', function () {
        var TestComponent = function () {
            var _a = __read(useAyanami(CountModule), 2), state = _a[0], actions = _a[1];
            var globalActions = useAyanamiDispatchers(GlobalModule);
            useEffect(function () {
                actions.waitUserAddCount();
                globalActions.fetchUser();
            }, []);
            return React.createElement("span", null, state.count);
        };
        var renderer = create(React.createElement(TestComponent, null));
        act(function () {
            renderer.update(React.createElement(TestComponent, null));
        });
        act(function () {
            timer.tick(ASYNC_DELAY_TIME);
            renderer.update(React.createElement(TestComponent, null));
        });
        expect(getCount()).toBe(1);
    });
    it('should be able to catch reducer error', function () {
        var error = new TypeError('whatever');
        var ReducerErrorModule = /** @class */ (function (_super) {
            __extends(ReducerErrorModule, _super);
            function ReducerErrorModule() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {};
                return _this;
            }
            ReducerErrorModule.prototype.throw = function () {
                throw error;
            };
            __decorate([
                Reducer(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", []),
                __metadata("design:returntype", void 0)
            ], ReducerErrorModule.prototype, "throw", null);
            ReducerErrorModule = __decorate([
                Module('ReducerError')
            ], ReducerErrorModule);
            return ReducerErrorModule;
        }(Ayanami));
        var spy = Sinon.spy();
        var Component = function () {
            var actions = useAyanamiDispatchers(ReducerErrorModule);
            try {
                actions.throw();
            }
            catch (e) {
                spy(e);
            }
            return React.createElement("div", null);
        };
        create(React.createElement(InjectableContext, null,
            React.createElement(Component, null)));
        expect(spy.args).toStrictEqual([[error]]);
    });
});
describe('Hooks spec: useAyanami with config', function () {
    var Component = function () {
        var _a = __read(useAyanami(CountModule, {
            selector: function (state) { return state.count; },
            mutateStateOnFirstRendering: function (state) {
                state.count = 2;
            },
        }), 2), count = _a[0], actions = _a[1];
        var addCount = useCallback(function () {
            actions.addCount();
        }, []);
        return (React.createElement("div", null,
            React.createElement("span", null, count),
            React.createElement("button", { id: "add-count", onClick: addCount }, "+")));
    };
    var testRenderer;
    var getCount;
    beforeEach(function () {
        testRenderer = create(React.createElement(Component, null));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        rootInjector.reset();
        rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should return selected state from store', function () {
        expect(getCount()).toBe(2);
    });
    it('should only mutate state on first rendering', function () {
        expect(getCount()).toBe(2);
        act(function () {
            testRenderer.root.findByType('button').props.onClick();
        });
        expect(getCount()).toBe(3);
    });
});
describe('Hooks spec: useAyanamiState', function () {
    var Component = function () {
        var count = useAyanamiState(CountModule, {
            selector: function (state) { return state.count; },
        });
        return (React.createElement("div", null,
            React.createElement("span", null, count)));
    };
    var testRenderer;
    var getCount;
    beforeEach(function () {
        testRenderer = create(React.createElement(Component, null));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        rootInjector.reset();
        rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should return selected state from store', function () {
        expect(getCount()).toBe(0);
    });
});
describe('Hooks spec: useAyanamiDispatchers', function () {
    var Component = function () {
        var state = useAyanamiState(CountModule);
        var actions = useAyanamiDispatchers(CountModule);
        var addCount = useCallback(function () {
            actions.addCount();
        }, []);
        return (React.createElement("div", null,
            React.createElement("span", null, state.count),
            React.createElement("button", { id: "add-count", onClick: addCount }, "+")));
    };
    var testRenderer;
    var getCount;
    beforeEach(function () {
        testRenderer = create(React.createElement(Component, null));
        getCount = function () {
            return parseInt(testRenderer.root.find(function (instance) { return instance.type === 'span'; }).children[0]);
        };
    });
    afterEach(function () {
        rootInjector.reset();
        rootInjector.addProviders([GlobalModule, CountModule]);
    });
    it('should return selected state from store', function () {
        expect(getCount()).toBe(0);
    });
    it('should dispatch effect', function () {
        act(function () {
            testRenderer.root.findByType('button').props.onClick();
        });
        expect(getCount()).toBe(1);
    });
});
