import { __decorate, __extends, __metadata, __read } from "tslib";
import 'reflect-metadata';
import { Test } from '@asuka/di';
import { Observable } from 'rxjs';
import { delay, map, withLatestFrom, combineLatest, skipWhile, take } from 'rxjs/operators';
import { useFakeTimers } from 'sinon';
import { AyanamiTestModule } from '../testbed';
import { Module, Ayanami, ImmerReducer, Effect } from '../core';
var ASYNC_DELAY_TIME = 1000;
var GlobalModule = /** @class */ (function (_super) {
    __extends(GlobalModule, _super);
    function GlobalModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = {
            user: '',
        };
        return _this;
    }
    GlobalModule.prototype.setUser = function (state, payload) {
        state.user = payload;
    };
    GlobalModule.prototype.fetchUser = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(ASYNC_DELAY_TIME), map(function () { return _this.getActions().setUser('global'); }));
    };
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, String]),
        __metadata("design:returntype", void 0)
    ], GlobalModule.prototype, "setUser", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], GlobalModule.prototype, "fetchUser", null);
    GlobalModule = __decorate([
        Module('Global')
    ], GlobalModule);
    return GlobalModule;
}(Ayanami));
var CountModule = /** @class */ (function (_super) {
    __extends(CountModule, _super);
    function CountModule(globalModule) {
        var _this = _super.call(this) || this;
        _this.globalModule = globalModule;
        _this.defaultState = {
            count: 0,
        };
        return _this;
    }
    CountModule.prototype.setCount = function (state, payload) {
        state.count = payload;
    };
    CountModule.prototype.addCount = function (payload$) {
        var _this = this;
        return payload$.pipe(withLatestFrom(this.state$), map(function (_a) {
            var _b = __read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.addCountAsync = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(ASYNC_DELAY_TIME), withLatestFrom(this.state$), map(function (_a) {
            var _b = __read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    CountModule.prototype.waitUserAddCount = function (payload$) {
        var _this = this;
        return payload$.pipe(combineLatest(this.globalModule.state$.pipe(skipWhile(function (v) { return !v.user; }), take(1)), this.state$.pipe(take(1))), map(function (_a) {
            var _b = __read(_a, 3), state = _b[2];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Number]),
        __metadata("design:returntype", void 0)
    ], CountModule.prototype, "setCount", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModule.prototype, "addCount", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModule.prototype, "addCountAsync", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", void 0)
    ], CountModule.prototype, "waitUserAddCount", null);
    CountModule = __decorate([
        Module('CountModule'),
        __metadata("design:paramtypes", [GlobalModule])
    ], CountModule);
    return CountModule;
}(Ayanami));
describe('AyanamiTestModule specs', function () {
    var globalStub;
    var countStub;
    var fakeTimer;
    beforeEach(function () {
        var testModule = Test.createTestingModule({
            TestModule: AyanamiTestModule,
            providers: [GlobalModule, CountModule],
        }).compile();
        globalStub = testModule.getAyanamiTestingStub(GlobalModule);
        countStub = testModule.getAyanamiTestingStub(CountModule);
        fakeTimer = useFakeTimers();
    });
    afterEach(function () {
        fakeTimer.restore();
    });
    it('should get state from global module', function () {
        var defaultState = globalStub.getState();
        expect(defaultState).toEqual({ user: '' });
    });
    it('should get state from count module', function () {
        var defaultState = countStub.getState();
        expect(defaultState).toEqual({ count: 0 });
    });
    it('should get dispatcher', function () {
        countStub.dispatcher.waitUserAddCount();
        globalStub.dispatcher.fetchUser();
        fakeTimer.tick(ASYNC_DELAY_TIME);
        expect(countStub.getState()).toEqual({ count: 1 });
    });
});
