import { __assign, __decorate, __extends, __metadata, __read } from "tslib";
import 'reflect-metadata';
import { Observable } from 'rxjs';
import { delay, map, withLatestFrom, takeUntil, tap } from 'rxjs/operators';
import { rootInjector } from '@asuka/di';
import * as Sinon from 'sinon';
import { noop } from 'rxjs';
import { Ayanami } from '../ayanami';
import { Module } from '../module.decorator';
import { Effect, Reducer, ImmerReducer, DefineAction } from '../decorators';
var TIME_TO_DELAY = 300;
var Counter = /** @class */ (function (_super) {
    __extends(Counter, _super);
    function Counter() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = {
            count: 1,
        };
        return _this;
    }
    Counter.prototype.setCountImmer = function (state, payload) {
        state.count = payload.valueOf();
    };
    Counter.prototype.addCountOneImmer = function (state) {
        state.count = state.count + 1;
    };
    Counter.prototype.addOne = function (state) {
        return __assign(__assign({}, state), { count: state.count + 1 });
    };
    Counter.prototype.setCount = function (state, payload) {
        return __assign(__assign({}, state), { count: payload });
    };
    Counter.prototype.asyncAddCount = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(TIME_TO_DELAY), map(function (payload) { return _this.getActions().asyncAddCountString("" + payload); }), takeUntil(this.dispose$));
    };
    Counter.prototype.asyncAddCountString = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(TIME_TO_DELAY), map(function (payload) { return _this.getActions().setCount(parseInt(payload)); }), takeUntil(this.getAction$().asyncAddCount));
    };
    Counter.prototype.asyncAddCountOne = function (payload$) {
        var _this = this;
        return payload$.pipe(delay(TIME_TO_DELAY), withLatestFrom(this.state$), map(function (_a) {
            var _b = __read(_a, 2), state = _b[1];
            return _this.getActions().setCount(state.count + 1);
        }));
    };
    Counter.prototype.effectWithPureSideEffect = function (payload$) {
        var _this = this;
        return payload$.pipe(tap(noop), map(function () { return _this.createNoopAction(); }));
    };
    __decorate([
        DefineAction(),
        __metadata("design:type", Observable)
    ], Counter.prototype, "dispose$", void 0);
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Date]),
        __metadata("design:returntype", void 0)
    ], Counter.prototype, "setCountImmer", null);
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], Counter.prototype, "addCountOneImmer", null);
    __decorate([
        Reducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], Counter.prototype, "addOne", null);
    __decorate([
        Reducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Number]),
        __metadata("design:returntype", void 0)
    ], Counter.prototype, "setCount", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], Counter.prototype, "asyncAddCount", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], Counter.prototype, "asyncAddCountString", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], Counter.prototype, "asyncAddCountOne", null);
    __decorate([
        Effect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", void 0)
    ], Counter.prototype, "effectWithPureSideEffect", null);
    Counter = __decorate([
        Module('counter')
    ], Counter);
    return Counter;
}(Ayanami));
describe('Ayanami Class', function () {
    var counter;
    beforeEach(function () {
        counter = rootInjector.resolveAndInstantiate(Counter);
    });
    describe('basic shape specs', function () {
        it('should dispatch reducer action', function () {
            expect(counter.getActions().setCount(1)).toStrictEqual({
                payload: 1,
                type: 'setCount',
            });
        });
        it('should dispatch effect action', function () {
            expect(counter.getActions().asyncAddCount(1)).toStrictEqual({
                payload: 1,
                type: 'asyncAddCount',
            });
        });
        it('should dispatch immer reducer action', function () {
            var payload = new Date();
            expect(counter.getActions().setCountImmer(payload)).toStrictEqual({
                payload: payload,
                type: 'setCountImmer',
            });
        });
        it('should dispatch reducer action with void payload', function () {
            expect(counter.getActions().addOne()).toStrictEqual({
                payload: undefined,
                type: 'addOne',
            });
        });
        it('should dispatch effect action with void payload', function () {
            expect(counter.getActions().asyncAddCountOne()).toStrictEqual({
                payload: undefined,
                type: 'asyncAddCountOne',
            });
        });
        it('should dispatch immer reducer action with void payload', function () {
            expect(counter.getActions().addCountOneImmer()).toStrictEqual({
                payload: undefined,
                type: 'addCountOneImmer',
            });
        });
        it('should dispatch action created by DefineAction', function () {
            expect(counter.getActions().dispose$()).toStrictEqual({
                payload: undefined,
                type: 'dispose$',
            });
        });
    });
    describe('complex shape specs', function () {
        it('should be able to create module with without epic', function () {
            var WithoutEpic = /** @class */ (function (_super) {
                __extends(WithoutEpic, _super);
                function WithoutEpic() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {
                        count: 0,
                    };
                    return _this;
                }
                WithoutEpic.prototype.set = function (state, payload) {
                    return __assign(__assign({}, state), { count: payload });
                };
                __decorate([
                    Reducer(),
                    __metadata("design:type", Function),
                    __metadata("design:paramtypes", [Object, Number]),
                    __metadata("design:returntype", void 0)
                ], WithoutEpic.prototype, "set", null);
                WithoutEpic = __decorate([
                    Module('WithoutEpic')
                ], WithoutEpic);
                return WithoutEpic;
            }(Ayanami));
            var onlyReducer = new WithoutEpic();
            var state = onlyReducer.createState();
            var actionCreator = onlyReducer.getActions();
            state.dispatch(actionCreator.set(1));
            expect(state.getState().count).toBe(1);
        });
        it('should be able to create module without reducer', function () {
            var WithoutReducer = /** @class */ (function (_super) {
                __extends(WithoutReducer, _super);
                function WithoutReducer() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {
                        count: 0,
                    };
                    return _this;
                }
                WithoutReducer.prototype.set = function (payload$) {
                    var _this = this;
                    return payload$.pipe(tap(noop), map(function () { return _this.createNoopAction(); }));
                };
                __decorate([
                    Effect(),
                    __metadata("design:type", Function),
                    __metadata("design:paramtypes", [Observable]),
                    __metadata("design:returntype", void 0)
                ], WithoutReducer.prototype, "set", null);
                WithoutReducer = __decorate([
                    Module('WithoutReducer')
                ], WithoutReducer);
                return WithoutReducer;
            }(Ayanami));
            var withoutReducer = new WithoutReducer();
            var state = withoutReducer.createState();
            var actionCreator = withoutReducer.getActions();
            state.dispatch(actionCreator.set(1));
            expect(state.getState().count).toBe(withoutReducer.defaultState.count);
        });
        it('should throw if module name conflict#1', function () {
            var fn = function () {
                var Module1 = /** @class */ (function (_super) {
                    __extends(Module1, _super);
                    function Module1() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module1 = __decorate([
                        Module('Module')
                    ], Module1);
                    return Module1;
                }(Ayanami));
                var Module2 = /** @class */ (function (_super) {
                    __extends(Module2, _super);
                    function Module2() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module2 = __decorate([
                        Module('Module')
                    ], Module2);
                    return Module2;
                }(Ayanami));
                return { Module1: Module1, Module2: Module2 };
            };
            expect(fn).toThrow();
        });
        it('should throw if module name conflict#2', function () {
            var fn = function () {
                var Module1 = /** @class */ (function (_super) {
                    __extends(Module1, _super);
                    function Module1() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module1 = __decorate([
                        Module('Module1')
                    ], Module1);
                    return Module1;
                }(Ayanami));
                var Module2 = /** @class */ (function (_super) {
                    __extends(Module2, _super);
                    function Module2() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module2 = __decorate([
                        Module('Module1')
                    ], Module2);
                    return Module2;
                }(Ayanami));
                return { Module1: Module1, Module2: Module2 };
            };
            expect(fn).toThrow();
        });
        it('should throw if config in module invalid', function () {
            var fn = function () {
                var Module1 = /** @class */ (function (_super) {
                    __extends(Module1, _super);
                    function Module1() {
                        var _this = _super !== null && _super.apply(this, arguments) || this;
                        _this.defaultState = {};
                        return _this;
                    }
                    Module1 = __decorate([
                        Module({})
                    ], Module1);
                    return Module1;
                }(Ayanami));
                return Module1;
            };
            expect(fn).toThrow();
        });
    });
    describe('dispatcher', function () {
        var state;
        var actionsDispatcher;
        var spy;
        var timer;
        beforeEach(function () {
            var actions = counter.getActions();
            state = counter.createState();
            actionsDispatcher = Object.keys(actions).reduce(function (acc, key) {
                acc[key] = function (p) {
                    var action = actions[key](p);
                    state.dispatch(action);
                    return action;
                };
                return acc;
            }, {});
            spy = Sinon.spy();
            state.subscribeAction(spy);
            timer = Sinon.useFakeTimers();
        });
        afterEach(function () {
            spy.resetHistory();
            timer.restore();
            state.unsubscribe();
        });
        it('should be able to dispatch reducer action by actions dispatcher #void', function () {
            var action = actionsDispatcher.addOne();
            var _a = __read(spy.args, 1), _b = __read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count + 1);
        });
        it('should be able to dispatch reducer action by actions dispatcher #param', function () {
            var newCount = 100;
            var action = actionsDispatcher.setCount(newCount);
            var _a = __read(spy.args, 1), _b = __read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(newCount);
        });
        it('should be able to dispatch immer reducer action by actions dispatcher #void', function () {
            var action = actionsDispatcher.addCountOneImmer();
            var _a = __read(spy.args, 1), _b = __read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count + 1);
        });
        it('should be able to dispatch immer reducer action by actions dispatcher #param', function () {
            var newCount = new Date();
            var action = actionsDispatcher.setCountImmer(newCount);
            var _a = __read(spy.args, 1), _b = __read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(newCount.valueOf());
        });
        it('should be able to dispatch epic action by actions dispatcher #void', function () {
            var action = actionsDispatcher.asyncAddCountOne();
            var _a = __read(spy.args, 1), _b = __read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count);
            timer.tick(TIME_TO_DELAY);
            var _c = __read(spy.args, 2), _d = __read(_c[1], 1), arg1 = _d[0];
            expect(arg1).toStrictEqual(counter.getActions().setCount(counter.defaultState.count + 1));
            expect(spy.callCount).toBe(2);
        });
        it('should be able to dispatch epic action by actions dispatcher #param', function () {
            var newCount = 100;
            var action = actionsDispatcher.asyncAddCount(newCount);
            var _a = __read(spy.args, 1), _b = __read(_a[0], 1), arg = _b[0];
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count);
            timer.tick(TIME_TO_DELAY);
            var _c = __read(spy.args, 2), _d = __read(_c[1], 1), arg1 = _d[0];
            expect(arg1).toStrictEqual(counter.getActions().asyncAddCountString("" + newCount));
            expect(spy.callCount).toBe(2);
        });
        it('should be able to dispose by other actions', function () {
            var newCount = 100;
            actionsDispatcher.asyncAddCount(newCount);
            expect(spy.callCount).toBe(1);
            actionsDispatcher.dispose$();
            timer.tick(TIME_TO_DELAY);
            expect(spy.callCount).toBe(2);
            expect(state.getState()).toStrictEqual(counter.defaultState);
        });
        it('should be able to dispatch noop action', function () {
            var action = actionsDispatcher.effectWithPureSideEffect();
            var _a = __read(spy.args, 2), _b = __read(_a[0], 1), arg1 = _b[0], _c = __read(_a[1], 1), arg2 = _c[0];
            expect(arg1).toStrictEqual(action);
            expect(typeof arg2.type).toBe('symbol');
            expect(spy.callCount).toBe(2);
            expect(state.getState()).toStrictEqual(counter.defaultState);
        });
    });
});
