import { __assign, __decorate, __extends, __metadata } from "tslib";
import 'reflect-metadata';
import { rootInjector } from '@asuka/di';
import * as Sinon from 'sinon';
import { empty, Observable } from 'rxjs';
import { delay, map } from 'rxjs/operators';
import { createState } from '../state';
import { Ayanami } from '../ayanami';
import { Reducer, Effect } from '../decorators';
import { Module } from '../module.decorator';
describe('Smoking tests', function () {
    it('Module should be able to work without effects', function () {
        var stateCreator = createState(function (state, action) {
            if (action.type === 'foo') {
                return __assign(__assign({}, state), { name: action.payload });
            }
            return state;
        }, function () { return empty(); }).stateCreator;
        var state = stateCreator({ name: 'bar' });
        var action = {
            type: 'foo',
            payload: 'foo',
            state: state,
        };
        state.dispatch(action);
        expect(state.getState().name).toBe(action.payload);
    });
    it('should be able to dispatch actions from the other module', function () {
        var asyncTimeToDelay = 2000;
        var timer = Sinon.useFakeTimers();
        var FooModule = /** @class */ (function (_super) {
            __extends(FooModule, _super);
            function FooModule() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {
                    foo: '1',
                };
                return _this;
            }
            FooModule.prototype.set = function (state, payload) {
                return __assign(__assign({}, state), { foo: payload });
            };
            __decorate([
                Reducer(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [Object, String]),
                __metadata("design:returntype", void 0)
            ], FooModule.prototype, "set", null);
            FooModule = __decorate([
                Module('Foo')
            ], FooModule);
            return FooModule;
        }(Ayanami));
        var BarModule = /** @class */ (function (_super) {
            __extends(BarModule, _super);
            function BarModule(fooModule) {
                var _this = _super.call(this) || this;
                _this.fooModule = fooModule;
                _this.defaultState = {};
                return _this;
            }
            BarModule.prototype.asyncSetFoo = function (payload$) {
                var _this = this;
                return payload$.pipe(delay(asyncTimeToDelay), map(function (payload) { return _this.fooModule.getActions().set(payload); }));
            };
            __decorate([
                Effect(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [Observable]),
                __metadata("design:returntype", void 0)
            ], BarModule.prototype, "asyncSetFoo", null);
            BarModule = __decorate([
                Module('Bar'),
                __metadata("design:paramtypes", [FooModule])
            ], BarModule);
            return BarModule;
        }(Ayanami));
        var fooModuleState = rootInjector.getInstance(FooModule);
        var barModuleState = rootInjector.getInstance(BarModule);
        var fooState = fooModuleState.createState();
        var barState = barModuleState.createState();
        var payload = 'whatever';
        var action = barModuleState.getActions().asyncSetFoo(payload);
        expect(action.payload).toBe(payload);
        barState.dispatch(action);
        timer.tick(asyncTimeToDelay);
        expect(fooState.getState().foo).toBe(payload);
        timer.restore();
        rootInjector.reset();
    });
});
