import { __values } from "tslib";
import { Subject, noop, ReplaySubject, Subscription, identity } from 'rxjs';
import { TERMINATE_ACTION } from '../ssr/terminate';
import { logStateAction } from '../redux-devtools-extension';
import { StateInterface } from './symbols';
export function createState(reducer, effect) {
    var action$ = new Subject();
    var _action$ = new Subject();
    var actionObservers = new Set();
    var state$ = new ReplaySubject(1);
    function stateCreator(defaultState, middleware, loadFromSSR) {
        var _a;
        if (middleware === void 0) { middleware = identity; }
        if (loadFromSSR === void 0) { loadFromSSR = false; }
        var state = Object.create(null);
        var appState = defaultState;
        function dispatch(action) {
            if (action.state !== state && action.type !== TERMINATE_ACTION.type) {
                action.state.dispatch(action);
                return;
            }
            var prevState = appState;
            var newState = reducer(prevState, action);
            if (newState !== prevState) {
                state$.next(newState);
            }
            logStateAction(action);
            action$.next(action);
            _action$.next(action);
        }
        var effect$ = effect(_action$, loadFromSSR);
        var subscription = new Subscription();
        subscription.add(middleware(effect$).subscribe(function (action) {
            try {
                dispatch(action);
            }
            catch (e) {
                action$.error(e);
            }
        }, function (err) {
            console.error(err);
        }));
        subscription.add(action$.subscribe(function (action) {
            var e_1, _a;
            try {
                for (var actionObservers_1 = __values(actionObservers), actionObservers_1_1 = actionObservers_1.next(); !actionObservers_1_1.done; actionObservers_1_1 = actionObservers_1.next()) {
                    var observer = actionObservers_1_1.value;
                    observer(action);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (actionObservers_1_1 && !actionObservers_1_1.done && (_a = actionObservers_1.return)) _a.call(actionObservers_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }, function (err) {
            _action$.error(err);
        }, function () {
            _action$.complete();
        }));
        subscription.add(state$.subscribe(function (state) {
            appState = state;
        }));
        state$.next(defaultState);
        Object.assign(state, (_a = {},
            _a[StateInterface] = state,
            _a.dispatch = dispatch,
            _a.state$ = state$,
            _a.getState = function () { return appState; },
            _a.subscribeAction = function (observer) {
                actionObservers.add(observer);
                return function () { return actionObservers.delete(observer); };
            },
            _a.unsubscribe = function () {
                action$.complete();
                state$.complete();
                subscription.unsubscribe();
                state.dispatch = noop;
                actionObservers.clear();
            },
            _a));
        return state;
    }
    return { stateCreator: stateCreator, action$: action$, state$: state$ };
}
