import { __read } from "tslib";
import React, { useContext } from 'react';
import { useInstance } from '@asuka/di';
import produce from 'immer';
import { map, distinctUntilChanged, skip } from 'rxjs/operators';
import { SSRSharedContext, SSRContext } from './ssr/ssr-context';
import { SSRStateCacheInstance } from './ssr/ssr-states';
import { oneShotCache } from './ssr/ssr-oneshot-cache';
import { SSR_LOADED_KEY } from './ssr/constants';
function _useAyanamiDispatchers(ayanami) {
    var appDispatcher = React.useMemo(function () {
        var state = ayanami.state;
        var actionsCreator = ayanami.getActions();
        return Object.keys(actionsCreator).reduce(function (acc, cur) {
            acc[cur] = function (payload) {
                var action = actionsCreator[cur](payload);
                state.dispatch(action);
            };
            return acc;
        }, Object.create(null));
    }, [ayanami]);
    return appDispatcher;
}
export function useAyanamiDispatchers(A) {
    var ayanami = _useState(A).ayanami;
    return _useAyanamiDispatchers(ayanami);
}
function _useAyanamiState(state, selector, mutateStateOnFirstRendering) {
    var _a = __read(React.useState(function () {
        var initialState = state.getState();
        if (typeof mutateStateOnFirstRendering === 'function') {
            initialState = produce(initialState, mutateStateOnFirstRendering);
            state.state$.next(initialState);
        }
        return selector && !Reflect.getMetadata(SSR_LOADED_KEY, state)
            ? selector(initialState)
            : initialState;
    }), 2), appState = _a[0], setState = _a[1];
    // https://reactjs.org/docs/hooks-faq.html#how-do-i-implement-getderivedstatefromprops
    // do not put subscribe in useEffect
    var subscription = React.useMemo(function () {
        return state.state$
            .pipe(skip(1), map(function (s) {
            if (Reflect.getMetadata(SSR_LOADED_KEY, state)) {
                Reflect.deleteMetadata(SSR_LOADED_KEY, state);
                return s;
            }
            else {
                return selector ? selector(s) : s;
            }
        }), distinctUntilChanged())
            .subscribe(setState);
    }, [state]);
    React.useEffect(function () { return function () { return subscription.unsubscribe(); }; }, [state]);
    return appState;
}
export function useAyanamiState(A, config) {
    var _a, _b;
    var state = _useState(A).state;
    return _useAyanamiState(state, (_a = config) === null || _a === void 0 ? void 0 : _a.selector, (_b = config) === null || _b === void 0 ? void 0 : _b.mutateStateOnFirstRendering);
}
export function useAyanami(A, config) {
    var _a, _b;
    var _c = _useState(A), ayanami = _c.ayanami, state = _c.state;
    var appState = _useAyanamiState(state, (_a = config) === null || _a === void 0 ? void 0 : _a.selector, (_b = config) === null || _b === void 0 ? void 0 : _b.mutateStateOnFirstRendering);
    var appDispatcher = _useAyanamiDispatchers(ayanami);
    return [appState, appDispatcher];
}
function _useState(A) {
    var ssrSharedContext = useContext(SSRSharedContext);
    var ssrContext = useContext(SSRContext);
    var ayanami = useInstance(A);
    var state = React.useMemo(function () {
        var _a;
        return SSRStateCacheInstance.has(ssrSharedContext, A)
            ? SSRStateCacheInstance.get(ssrSharedContext, A)
            : ssrContext
                ? (_a = oneShotCache.consume(ssrContext, A), (_a !== null && _a !== void 0 ? _a : ayanami.createState())) : ayanami.createState();
    }, [ayanami, ssrContext, ssrSharedContext]);
    return { ayanami: ayanami, state: state };
}
