import { __extends } from "tslib";
import Sinon from 'sinon';
import { noop } from 'rxjs';
import { SSRStateCache } from '../ssr-states';
import { Ayanami } from '../../core';
describe('SSRStateCacheInstance specs:', function () {
    var SSRStateCacheInstance;
    var A = /** @class */ (function (_super) {
        __extends(A, _super);
        function A() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.defaultState = {};
            return _this;
        }
        return A;
    }(Ayanami));
    var ctx = {};
    var state = Object.create({
        unsubscribe: noop,
    });
    beforeEach(function () {
        SSRStateCacheInstance = new SSRStateCache();
        SSRStateCacheInstance.set(ctx, A, state);
    });
    it('has should work', function () {
        expect(SSRStateCacheInstance.has(ctx, A)).toBe(true);
    });
    it('get existed should work', function () {
        expect(SSRStateCacheInstance.get(ctx, A)).toBe(state);
    });
    it('get not existed should return undefined', function () {
        expect(SSRStateCacheInstance.get({}, A)).toBe(undefined);
    });
    it('should clear all if pool size smaller than cache size', function () {
        SSRStateCacheInstance.setPoolSize(1);
        expect(SSRStateCacheInstance.get({}, A)).toBe(undefined);
        var newCtx = {};
        var newState = Object.create(null);
        SSRStateCacheInstance.set(newCtx, A, newState);
        expect(SSRStateCacheInstance.get(newCtx, A)).toBe(newState);
        SSRStateCacheInstance.set(ctx, A, state);
        expect(SSRStateCacheInstance.get(newCtx, A)).toBe(undefined);
    });
    it('should delete all states after cleanupStateByContext', function () {
        var spy = Sinon.spy();
        var newState = Object.create({
            unsubscribe: spy,
        });
        var B = /** @class */ (function (_super) {
            __extends(B, _super);
            function B() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {};
                return _this;
            }
            return B;
        }(Ayanami));
        SSRStateCacheInstance.set(ctx, B, newState);
        expect(SSRStateCacheInstance.get(ctx, B)).toBe(newState);
        SSRStateCacheInstance.cleanupStateByContext(ctx);
        expect(SSRStateCacheInstance.get(ctx, A)).toBeUndefined();
        expect(SSRStateCacheInstance.get(ctx, B)).toBeUndefined();
        expect(spy.callCount).toBe(1);
    });
    it('should do nothing when cleanupStateByContext non-existed ctx', function () {
        SSRStateCacheInstance.cleanupStateByContext({});
        expect(SSRStateCacheInstance.get(ctx, A)).toBe(state);
    });
});
