import { __assign, __awaiter, __decorate, __extends, __generator, __metadata, __read } from "tslib";
import 'reflect-metadata';
import React from 'react';
import { Observable, timer } from 'rxjs';
import { endWith, switchMap, map, mergeMap, flatMap, withLatestFrom } from 'rxjs/operators';
import { rootInjector } from '@asuka/di';
import { renderToString } from 'react-dom/server';
import { create, act } from 'react-test-renderer';
import uniqueId from 'lodash/uniqueId';
import { TERMINATE_ACTION, emitSSREffects, SSREffect } from '../index';
import { Ayanami, ImmerReducer, Module, Effect, Reducer } from '../../core';
import { useAyanami, useAyanamiState } from '../../hooks';
import { GLOBAL_KEY } from '../constants';
import { SSRSharedContext, SSRContext } from '../ssr-context';
import { SSRStateCacheInstance } from '../ssr-states';
var CountModel = /** @class */ (function (_super) {
    __extends(CountModel, _super);
    function CountModel() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = { count: 0, name: '' };
        return _this;
    }
    CountModel.prototype.setCount = function (state, count) {
        state.count = count;
    };
    CountModel.prototype.setName = function (state, name) {
        state.name = name;
    };
    CountModel.prototype.getCount = function (payload$) {
        var _this = this;
        return payload$.pipe(flatMap(function () {
            return timer(20).pipe(map(function () { return _this.getActions().setCount(1); }), endWith(TERMINATE_ACTION));
        }));
    };
    CountModel.prototype.skippedEffect = function (payload$) {
        var _this = this;
        return payload$.pipe(switchMap(function (name) {
            return timer(20).pipe(map(function () { return _this.getActions().setName(name); }), endWith(TERMINATE_ACTION));
        }));
    };
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Number]),
        __metadata("design:returntype", void 0)
    ], CountModel.prototype, "setCount", null);
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, String]),
        __metadata("design:returntype", void 0)
    ], CountModel.prototype, "setName", null);
    __decorate([
        SSREffect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModel.prototype, "getCount", null);
    __decorate([
        SSREffect({
            payloadGetter: function (ctx, skip) { return ctx.url || skip(); },
            skipFirstClientDispatch: false,
        }),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], CountModel.prototype, "skippedEffect", null);
    CountModel = __decorate([
        Module('CountModel')
    ], CountModel);
    return CountModel;
}(Ayanami));
var TipModel = /** @class */ (function (_super) {
    __extends(TipModel, _super);
    function TipModel() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.defaultState = { tip: '' };
        return _this;
    }
    TipModel.prototype.setTip = function (state, tip) {
        state.tip = tip;
    };
    TipModel.prototype.getTip = function (payload$) {
        var _this = this;
        return payload$.pipe(mergeMap(function () {
            return timer(1).pipe(map(function () { return _this.getActions().setTip('tip'); }), endWith(TERMINATE_ACTION));
        }));
    };
    __decorate([
        ImmerReducer(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, String]),
        __metadata("design:returntype", void 0)
    ], TipModel.prototype, "setTip", null);
    __decorate([
        SSREffect(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Observable]),
        __metadata("design:returntype", Observable)
    ], TipModel.prototype, "getTip", null);
    TipModel = __decorate([
        Module('TipModel')
    ], TipModel);
    return TipModel;
}(Ayanami));
var Component = function () {
    var _a = __read(useAyanami(CountModel), 2), state = _a[0], actions = _a[1];
    React.useEffect(function () {
        actions.setName('new name');
    }, []);
    return (React.createElement(React.Fragment, null,
        React.createElement("span", null, state.count)));
};
var ComponentWithSelector = function () {
    var _a = __read(useAyanami(CountModel, {
        selector: function (s) { return ({
            count: s.count + 1,
        }); },
    }), 2), state = _a[0], actions = _a[1];
    React.useEffect(function () {
        actions.setName('new name');
    }, []);
    return (React.createElement(React.Fragment, null,
        React.createElement("span", null, state.count)));
};
describe('SSR specs:', function () {
    beforeEach(function () {
        SSRStateCacheInstance.setPoolSize(100);
        rootInjector.addProviders([CountModel, TipModel]);
    });
    afterEach(function () {
        SSRStateCacheInstance.setPoolSize(0);
        rootInjector.reset();
    });
    it('should throw if module name not given', function () {
        function generateException() {
            var ErrorModel = /** @class */ (function (_super) {
                __extends(ErrorModel, _super);
                function ErrorModel() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {};
                    return _this;
                }
                ErrorModel = __decorate([
                    (Module())
                ], ErrorModel);
                return ErrorModel;
            }(Ayanami));
            return ErrorModel;
        }
        expect(generateException).toThrow();
    });
    it('should pass valid module name', function () {
        var Model = /** @class */ (function (_super) {
            __extends(Model, _super);
            function Model() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {};
                return _this;
            }
            Model = __decorate([
                Module('1')
            ], Model);
            return Model;
        }(Ayanami));
        var Model2 = /** @class */ (function (_super) {
            __extends(Model2, _super);
            function Model2() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.defaultState = {};
                return _this;
            }
            Model2 = __decorate([
                Module('2')
            ], Model2);
            return Model2;
        }(Ayanami));
        function generateException1() {
            var ErrorModel1 = /** @class */ (function (_super) {
                __extends(ErrorModel1, _super);
                function ErrorModel1() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {};
                    return _this;
                }
                ErrorModel1 = __decorate([
                    Module('1')
                ], ErrorModel1);
                return ErrorModel1;
            }(Ayanami));
            return ErrorModel1;
        }
        function generateException2() {
            var ErrorModel2 = /** @class */ (function (_super) {
                __extends(ErrorModel2, _super);
                function ErrorModel2() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {};
                    return _this;
                }
                ErrorModel2 = __decorate([
                    Module('1')
                ], ErrorModel2);
                return ErrorModel2;
            }(Ayanami));
            return { ErrorModel2: ErrorModel2 };
        }
        function generateException3() {
            var ErrorModel = /** @class */ (function (_super) {
                __extends(ErrorModel, _super);
                function ErrorModel() {
                    var _this = _super !== null && _super.apply(this, arguments) || this;
                    _this.defaultState = {};
                    return _this;
                }
                ErrorModel = __decorate([
                    (Module())
                ], ErrorModel);
                return ErrorModel;
            }(Ayanami));
            return ErrorModel;
        }
        expect(Model).not.toBe(undefined);
        expect(Model2).not.toBe(undefined);
        expect(generateException1).toThrow();
        expect(generateException2).toThrow();
        expect(generateException3).toThrow();
    });
    it('should run ssr effects', function () { return __awaiter(void 0, void 0, void 0, function () {
        var state, moduleState;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, emitSSREffects({ url: 'name' }, [CountModel])];
                case 1:
                    state = _a.sent();
                    moduleState = state['CountModel'];
                    expect(moduleState).not.toBe(undefined);
                    expect(moduleState.count).toBe(1);
                    expect(moduleState.name).toBe('name');
                    expect(state).toMatchSnapshot();
                    return [2 /*return*/];
            }
        });
    }); });
    it('should skip effect if it returns SKIP_SYMBOL', function () { return __awaiter(void 0, void 0, void 0, function () {
        var state, moduleState;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, emitSSREffects({}, [CountModel])];
                case 1:
                    state = _a.sent();
                    moduleState = state['CountModel'];
                    expect(moduleState.name).toBe('');
                    expect(state).toMatchSnapshot();
                    return [2 /*return*/];
            }
        });
    }); });
    it('should return right state in hooks', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, html;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    req = {};
                    return [4 /*yield*/, emitSSREffects(req, [CountModel])];
                case 1:
                    _a.sent();
                    html = renderToString(React.createElement(SSRContext.Provider, { value: req },
                        React.createElement(Component, null)));
                    expect(html).toContain('<span>1</span>');
                    expect(html).toMatchSnapshot();
                    return [2 /*return*/];
            }
        });
    }); });
    it('should restore state from global', function () {
        // @ts-ignore
        global[GLOBAL_KEY] = {
            CountModel: {
                count: 1,
                name: '',
            },
        };
        var testRenderer = create(React.createElement(Component, null));
        act(function () {
            testRenderer.update(React.createElement(Component, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('1');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should restore state from global #with selector', function () {
        // @ts-ignore
        global[GLOBAL_KEY] = {
            CountModel: {
                count: 10,
                name: '',
            },
        };
        var testRenderer = create(React.createElement(ComponentWithSelector, null));
        act(function () {
            testRenderer.update(React.createElement(ComponentWithSelector, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('10');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should not restore state from global if state is null', function () {
        // @ts-ignore
        global[GLOBAL_KEY] = {
            OtherModule: {
                count: 10,
                name: '',
            },
        };
        var testRenderer = create(React.createElement(ComponentWithSelector, null));
        act(function () {
            testRenderer.update(React.createElement(ComponentWithSelector, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('1');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should restore and skip first action on client side', function () {
        var Component = function () {
            var _a = __read(useAyanami(CountModel), 2), state = _a[0], actions = _a[1];
            React.useEffect(function () {
                actions.getCount();
            }, []);
            return (React.createElement(React.Fragment, null,
                React.createElement("span", null, state.count)));
        };
        // @ts-ignore
        global[GLOBAL_KEY] = {
            CountModel: {
                count: 2,
                name: '',
            },
        };
        var testRenderer = create(React.createElement(Component, null));
        act(function () {
            testRenderer.update(React.createElement(Component, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('2');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should support concurrency', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, Promise.all([
                    emitSSREffects({ url: 'name1' }, [CountModel, TipModel], 'concurrency1'),
                    emitSSREffects({ url: 'name2' }, [CountModel, TipModel], 'concurrency2'),
                ]).then(function (_a) {
                    var _b = __read(_a, 2), result1 = _b[0], result2 = _b[1];
                    expect(result1['CountModel'].name).toBe('name1');
                    expect(result2['CountModel'].name).toBe('name2');
                    expect({
                        firstRequest: result1,
                        secondRequest: result2,
                    }).toMatchSnapshot();
                })];
        });
    }); });
    it('should timeout', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, reqContext;
        return __generator(this, function (_a) {
            req = {};
            reqContext = uniqueId();
            return [2 /*return*/, emitSSREffects(req, [CountModel], reqContext, 0).catch(function (e) {
                    expect(e.message).toBe('Terminate timeout');
                })];
        });
    }); });
    it('should resolve empty object if no modules provided', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, state;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    req = {};
                    return [4 /*yield*/, emitSSREffects(req, [])];
                case 1:
                    state = _a.sent();
                    expect(state).toStrictEqual({});
                    return [2 /*return*/];
            }
        });
    }); });
    it('should do nothing if Module contains no SSREffects', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, WithoutSSRModule, state;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    req = {};
                    WithoutSSRModule = /** @class */ (function (_super) {
                        __extends(WithoutSSRModule, _super);
                        function WithoutSSRModule() {
                            var _this = _super !== null && _super.apply(this, arguments) || this;
                            _this.defaultState = {
                                count: 0,
                            };
                            return _this;
                        }
                        WithoutSSRModule.prototype.set = function (state, payload) {
                            state.count = payload;
                        };
                        WithoutSSRModule.prototype.addOne = function (payload$) {
                            var _this = this;
                            return payload$.pipe(withLatestFrom(this.state$), map(function (_a) {
                                var _b = __read(_a, 2), count = _b[1].count;
                                return _this.getActions().set(count + 1);
                            }));
                        };
                        __decorate([
                            ImmerReducer(),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", [Object, Number]),
                            __metadata("design:returntype", void 0)
                        ], WithoutSSRModule.prototype, "set", null);
                        __decorate([
                            Effect(),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", [Observable]),
                            __metadata("design:returntype", Observable)
                        ], WithoutSSRModule.prototype, "addOne", null);
                        WithoutSSRModule = __decorate([
                            Module('WithoutSSR')
                        ], WithoutSSRModule);
                        return WithoutSSRModule;
                    }(Ayanami));
                    return [4 /*yield*/, emitSSREffects(req, [WithoutSSRModule])];
                case 1:
                    state = _a.sent();
                    expect(state).toStrictEqual({});
                    return [2 /*return*/];
            }
        });
    }); });
    it('should throw error if runEffects error', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, error, SSRErrorModule, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    req = {};
                    error = new TypeError('whatever');
                    SSRErrorModule = /** @class */ (function (_super) {
                        __extends(SSRErrorModule, _super);
                        function SSRErrorModule() {
                            var _this = _super !== null && _super.apply(this, arguments) || this;
                            _this.defaultState = {
                                count: 0,
                            };
                            return _this;
                        }
                        SSRErrorModule.prototype.addOne = function (payload$) {
                            return payload$.pipe(withLatestFrom(this.state$), map(function () {
                                throw error;
                            }));
                        };
                        __decorate([
                            SSREffect(),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", [Observable]),
                            __metadata("design:returntype", Observable)
                        ], SSRErrorModule.prototype, "addOne", null);
                        SSRErrorModule = __decorate([
                            Module('ErrorModule')
                        ], SSRErrorModule);
                        return SSRErrorModule;
                    }(Ayanami));
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, emitSSREffects(req, [SSRErrorModule])];
                case 2:
                    _a.sent();
                    throw new TypeError('Unreachable code path');
                case 3:
                    e_1 = _a.sent();
                    expect(e_1).toBe(error);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    it('should throw error if reducer throw', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, error, SSRErrorModule, e_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    req = {};
                    error = new TypeError('whatever');
                    SSRErrorModule = /** @class */ (function (_super) {
                        __extends(SSRErrorModule, _super);
                        function SSRErrorModule() {
                            var _this = _super !== null && _super.apply(this, arguments) || this;
                            _this.defaultState = {
                                count: 0,
                            };
                            return _this;
                        }
                        SSRErrorModule.prototype.set = function () {
                            throw error;
                        };
                        SSRErrorModule.prototype.addOne = function (payload$) {
                            var _this = this;
                            return payload$.pipe(withLatestFrom(this.state$), map(function () { return _this.getActions().set(); }));
                        };
                        __decorate([
                            Reducer(),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", []),
                            __metadata("design:returntype", void 0)
                        ], SSRErrorModule.prototype, "set", null);
                        __decorate([
                            SSREffect(),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", [Observable]),
                            __metadata("design:returntype", Observable)
                        ], SSRErrorModule.prototype, "addOne", null);
                        SSRErrorModule = __decorate([
                            Module('ErrorReducerModule')
                        ], SSRErrorModule);
                        return SSRErrorModule;
                    }(Ayanami));
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, emitSSREffects(req, [SSRErrorModule])];
                case 2:
                    _a.sent();
                    throw new TypeError('Unreachable code path');
                case 3:
                    e_2 = _a.sent();
                    expect(e_2).toBe(error);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    it('should throw error if payloadGetter throw', function () { return __awaiter(void 0, void 0, void 0, function () {
        var req, error, SSRErrorModule, e_3;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    req = {};
                    error = new TypeError('whatever');
                    SSRErrorModule = /** @class */ (function (_super) {
                        __extends(SSRErrorModule, _super);
                        function SSRErrorModule() {
                            var _this = _super !== null && _super.apply(this, arguments) || this;
                            _this.defaultState = {
                                count: 0,
                            };
                            return _this;
                        }
                        SSRErrorModule.prototype.set = function (state, payload) {
                            return __assign(__assign({}, state), { count: payload });
                        };
                        SSRErrorModule.prototype.addOne = function (payload$) {
                            var _this = this;
                            return payload$.pipe(withLatestFrom(this.state$), map(function (_a) {
                                var _b = __read(_a, 2), state = _b[1];
                                return _this.getActions().set(state.count + 1);
                            }));
                        };
                        __decorate([
                            Reducer(),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", [Object, Number]),
                            __metadata("design:returntype", void 0)
                        ], SSRErrorModule.prototype, "set", null);
                        __decorate([
                            SSREffect({
                                payloadGetter: function () {
                                    throw error;
                                },
                            }),
                            __metadata("design:type", Function),
                            __metadata("design:paramtypes", [Observable]),
                            __metadata("design:returntype", Observable)
                        ], SSRErrorModule.prototype, "addOne", null);
                        SSRErrorModule = __decorate([
                            Module('ErrorMiddlewareModule')
                        ], SSRErrorModule);
                        return SSRErrorModule;
                    }(Ayanami));
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, emitSSREffects(req, [SSRErrorModule])];
                case 2:
                    _a.sent();
                    throw new TypeError('Unreachable code path');
                case 3:
                    e_3 = _a.sent();
                    expect(e_3).toBe(error);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    it('should reuse state if provider context', function () { return __awaiter(void 0, void 0, void 0, function () {
        var requestId, req1, req2, SharedComponent1, SharedComponent2, cachedState, result1, result2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    requestId = uniqueId();
                    req1 = {};
                    req2 = {};
                    return [4 /*yield*/, emitSSREffects(req1, [CountModel], requestId)];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, emitSSREffects(req2, [CountModel], requestId)];
                case 2:
                    _a.sent();
                    SharedComponent1 = function () {
                        var state = useAyanamiState(CountModel);
                        return (React.createElement(React.Fragment, null,
                            React.createElement("span", null, state.count)));
                    };
                    SharedComponent2 = function () {
                        var state = useAyanamiState(CountModel);
                        return (React.createElement(React.Fragment, null,
                            React.createElement("span", null, state.count)));
                    };
                    cachedState = SSRStateCacheInstance.get(requestId, CountModel);
                    expect(cachedState.getState().count).toBe(1);
                    result1 = renderToString(React.createElement(SSRSharedContext.Provider, { value: requestId },
                        React.createElement(SharedComponent1, null)));
                    result2 = renderToString(React.createElement(SSRSharedContext.Provider, { value: requestId },
                        React.createElement(SharedComponent2, null)));
                    expect(result1).toBe(result2);
                    return [2 /*return*/];
            }
        });
    }); });
});
