import { __values } from "tslib";
import { StateInterface } from '../core/symbols';
var SSROneShotCache = /** @class */ (function () {
    function SSROneShotCache() {
        this.cache = new Map();
        this.activedCtx = new Set();
    }
    SSROneShotCache.prototype.store = function (ctx, ctor, instance) {
        if (this.cache.has(ctx)) {
            this.cache.get(ctx).set(ctor, instance);
        }
        else {
            var cache = new Map();
            cache.set(ctor, instance);
            this.cache.set(ctx, cache);
        }
    };
    SSROneShotCache.prototype.consume = function (ctx, ctor) {
        var _this = this;
        var _a;
        var instance = (_a = this.cache.get(ctx)) === null || _a === void 0 ? void 0 : _a.get(ctor);
        if (!this.activedCtx.has(ctx) && instance) {
            this.activedCtx.add(ctx);
            Promise.resolve().then(function () {
                _this.destroy(ctx);
            });
        }
        return instance;
    };
    SSROneShotCache.prototype.destroy = function (ctx) {
        var e_1, _a;
        if (this.activedCtx.has(ctx)) {
            this.activedCtx.delete(ctx);
            if (this.cache.has(ctx)) {
                try {
                    for (var _b = __values(this.cache.get(ctx).values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var instance = _c.value;
                        if (instance && instance[StateInterface]) {
                            instance[StateInterface].unsubscribe();
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                this.cache.delete(ctx);
            }
        }
    };
    return SSROneShotCache;
}());
export { SSROneShotCache };
export var oneShotCache = new SSROneShotCache();
