import { __decorate, __metadata } from "tslib";
import 'reflect-metadata';
import { Test } from '@asuka/di';
import { Observable } from 'rxjs';
import { delay, map, withLatestFrom, combineLatest, skipWhile, take } from 'rxjs/operators';
import { useFakeTimers } from 'sinon';
import { AyanamiTestModule } from '../testbed';
import { Module, Ayanami, ImmerReducer, Effect } from '../core';
const ASYNC_DELAY_TIME = 1000;
let GlobalModule = class GlobalModule extends Ayanami {
    constructor() {
        super(...arguments);
        this.defaultState = {
            user: '',
        };
    }
    setUser(state, payload) {
        state.user = payload;
    }
    fetchUser(payload$) {
        return payload$.pipe(delay(ASYNC_DELAY_TIME), map(() => this.getActions().setUser('global')));
    }
};
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", void 0)
], GlobalModule.prototype, "setUser", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], GlobalModule.prototype, "fetchUser", null);
GlobalModule = __decorate([
    Module('Global')
], GlobalModule);
let CountModule = class CountModule extends Ayanami {
    constructor(globalModule) {
        super();
        this.globalModule = globalModule;
        this.defaultState = {
            count: 0,
        };
    }
    setCount(state, payload) {
        state.count = payload;
    }
    addCount(payload$) {
        return payload$.pipe(withLatestFrom(this.state$), map(([, state]) => this.getActions().setCount(state.count + 1)));
    }
    addCountAsync(payload$) {
        return payload$.pipe(delay(ASYNC_DELAY_TIME), withLatestFrom(this.state$), map(([, state]) => this.getActions().setCount(state.count + 1)));
    }
    waitUserAddCount(payload$) {
        return payload$.pipe(combineLatest(this.globalModule.state$.pipe(skipWhile((v) => !v.user), take(1)), this.state$.pipe(take(1))), map(([, , state]) => this.getActions().setCount(state.count + 1)));
    }
};
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Number]),
    __metadata("design:returntype", void 0)
], CountModule.prototype, "setCount", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], CountModule.prototype, "addCount", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], CountModule.prototype, "addCountAsync", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", void 0)
], CountModule.prototype, "waitUserAddCount", null);
CountModule = __decorate([
    Module('CountModule'),
    __metadata("design:paramtypes", [GlobalModule])
], CountModule);
describe('AyanamiTestModule specs', () => {
    let globalStub;
    let countStub;
    let fakeTimer;
    beforeEach(() => {
        const testModule = Test.createTestingModule({
            TestModule: AyanamiTestModule,
            providers: [GlobalModule, CountModule],
        }).compile();
        globalStub = testModule.getAyanamiTestingStub(GlobalModule);
        countStub = testModule.getAyanamiTestingStub(CountModule);
        fakeTimer = useFakeTimers();
    });
    afterEach(() => {
        fakeTimer.restore();
    });
    it('should get state from global module', () => {
        const defaultState = globalStub.getState();
        expect(defaultState).toEqual({ user: '' });
    });
    it('should get state from count module', () => {
        const defaultState = countStub.getState();
        expect(defaultState).toEqual({ count: 0 });
    });
    it('should get dispatcher', () => {
        countStub.dispatcher.waitUserAddCount();
        globalStub.dispatcher.fetchUser();
        fakeTimer.tick(ASYNC_DELAY_TIME);
        expect(countStub.getState()).toEqual({ count: 1 });
    });
});
