import { __decorate, __metadata } from "tslib";
import 'reflect-metadata';
import { Observable } from 'rxjs';
import { delay, map, withLatestFrom, takeUntil, tap } from 'rxjs/operators';
import { rootInjector } from '@asuka/di';
import * as Sinon from 'sinon';
import { noop } from 'rxjs';
import { Ayanami } from '../ayanami';
import { Module } from '../module.decorator';
import { Effect, Reducer, ImmerReducer, DefineAction } from '../decorators';
const TIME_TO_DELAY = 300;
let Counter = class Counter extends Ayanami {
    constructor() {
        super(...arguments);
        this.defaultState = {
            count: 1,
        };
    }
    setCountImmer(state, payload) {
        state.count = payload.valueOf();
    }
    addCountOneImmer(state) {
        state.count = state.count + 1;
    }
    addOne(state) {
        return { ...state, count: state.count + 1 };
    }
    setCount(state, payload) {
        return { ...state, count: payload };
    }
    asyncAddCount(payload$) {
        return payload$.pipe(delay(TIME_TO_DELAY), map((payload) => this.getActions().asyncAddCountString(`${payload}`)), takeUntil(this.dispose$));
    }
    asyncAddCountString(payload$) {
        return payload$.pipe(delay(TIME_TO_DELAY), map((payload) => this.getActions().setCount(parseInt(payload))), takeUntil(this.getAction$().asyncAddCount));
    }
    asyncAddCountOne(payload$) {
        return payload$.pipe(delay(TIME_TO_DELAY), withLatestFrom(this.state$), map(([, state]) => this.getActions().setCount(state.count + 1)));
    }
    effectWithPureSideEffect(payload$) {
        return payload$.pipe(tap(noop), map(() => this.createNoopAction()));
    }
};
__decorate([
    DefineAction(),
    __metadata("design:type", Observable)
], Counter.prototype, "dispose$", void 0);
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Date]),
    __metadata("design:returntype", void 0)
], Counter.prototype, "setCountImmer", null);
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], Counter.prototype, "addCountOneImmer", null);
__decorate([
    Reducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], Counter.prototype, "addOne", null);
__decorate([
    Reducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Number]),
    __metadata("design:returntype", void 0)
], Counter.prototype, "setCount", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], Counter.prototype, "asyncAddCount", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], Counter.prototype, "asyncAddCountString", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], Counter.prototype, "asyncAddCountOne", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", void 0)
], Counter.prototype, "effectWithPureSideEffect", null);
Counter = __decorate([
    Module('counter')
], Counter);
describe('Ayanami Class', () => {
    let counter;
    beforeEach(() => {
        counter = rootInjector.resolveAndInstantiate(Counter);
    });
    describe('basic shape specs', () => {
        it('should dispatch reducer action', () => {
            expect(counter.getActions().setCount(1)).toStrictEqual({
                payload: 1,
                type: 'setCount',
            });
        });
        it('should dispatch effect action', () => {
            expect(counter.getActions().asyncAddCount(1)).toStrictEqual({
                payload: 1,
                type: 'asyncAddCount',
            });
        });
        it('should dispatch immer reducer action', () => {
            const payload = new Date();
            expect(counter.getActions().setCountImmer(payload)).toStrictEqual({
                payload,
                type: 'setCountImmer',
            });
        });
        it('should dispatch reducer action with void payload', () => {
            expect(counter.getActions().addOne()).toStrictEqual({
                payload: undefined,
                type: 'addOne',
            });
        });
        it('should dispatch effect action with void payload', () => {
            expect(counter.getActions().asyncAddCountOne()).toStrictEqual({
                payload: undefined,
                type: 'asyncAddCountOne',
            });
        });
        it('should dispatch immer reducer action with void payload', () => {
            expect(counter.getActions().addCountOneImmer()).toStrictEqual({
                payload: undefined,
                type: 'addCountOneImmer',
            });
        });
        it('should dispatch action created by DefineAction', () => {
            expect(counter.getActions().dispose$()).toStrictEqual({
                payload: undefined,
                type: 'dispose$',
            });
        });
    });
    describe('complex shape specs', () => {
        it('should be able to create module with without epic', () => {
            let WithoutEpic = class WithoutEpic extends Ayanami {
                constructor() {
                    super(...arguments);
                    this.defaultState = {
                        count: 0,
                    };
                }
                set(state, payload) {
                    return { ...state, count: payload };
                }
            };
            __decorate([
                Reducer(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [Object, Number]),
                __metadata("design:returntype", void 0)
            ], WithoutEpic.prototype, "set", null);
            WithoutEpic = __decorate([
                Module('WithoutEpic')
            ], WithoutEpic);
            const onlyReducer = new WithoutEpic();
            const state = onlyReducer.createState();
            const actionCreator = onlyReducer.getActions();
            state.dispatch(actionCreator.set(1));
            expect(state.getState().count).toBe(1);
        });
        it('should be able to create module without reducer', () => {
            let WithoutReducer = class WithoutReducer extends Ayanami {
                constructor() {
                    super(...arguments);
                    this.defaultState = {
                        count: 0,
                    };
                }
                set(payload$) {
                    return payload$.pipe(tap(noop), map(() => this.createNoopAction()));
                }
            };
            __decorate([
                Effect(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [Observable]),
                __metadata("design:returntype", void 0)
            ], WithoutReducer.prototype, "set", null);
            WithoutReducer = __decorate([
                Module('WithoutReducer')
            ], WithoutReducer);
            const withoutReducer = new WithoutReducer();
            const state = withoutReducer.createState();
            const actionCreator = withoutReducer.getActions();
            state.dispatch(actionCreator.set(1));
            expect(state.getState().count).toBe(withoutReducer.defaultState.count);
        });
        it('should throw if module name conflict#1', () => {
            const fn = () => {
                let Module1 = class Module1 extends Ayanami {
                    constructor() {
                        super(...arguments);
                        this.defaultState = {};
                    }
                };
                Module1 = __decorate([
                    Module('Module')
                ], Module1);
                let Module2 = class Module2 extends Ayanami {
                    constructor() {
                        super(...arguments);
                        this.defaultState = {};
                    }
                };
                Module2 = __decorate([
                    Module('Module')
                ], Module2);
                return { Module1, Module2 };
            };
            expect(fn).toThrow();
        });
        it('should throw if module name conflict#2', () => {
            const fn = () => {
                let Module1 = class Module1 extends Ayanami {
                    constructor() {
                        super(...arguments);
                        this.defaultState = {};
                    }
                };
                Module1 = __decorate([
                    Module('Module1')
                ], Module1);
                let Module2 = class Module2 extends Ayanami {
                    constructor() {
                        super(...arguments);
                        this.defaultState = {};
                    }
                };
                Module2 = __decorate([
                    Module('Module1')
                ], Module2);
                return { Module1, Module2 };
            };
            expect(fn).toThrow();
        });
        it('should throw if config in module invalid', () => {
            const fn = () => {
                let Module1 = class Module1 extends Ayanami {
                    constructor() {
                        super(...arguments);
                        this.defaultState = {};
                    }
                };
                Module1 = __decorate([
                    Module({})
                ], Module1);
                return Module1;
            };
            expect(fn).toThrow();
        });
    });
    describe('dispatcher', () => {
        let state;
        let actionsDispatcher;
        let spy;
        let timer;
        beforeEach(() => {
            const actions = counter.getActions();
            state = counter.createState();
            actionsDispatcher = Object.keys(actions).reduce((acc, key) => {
                acc[key] = (p) => {
                    const action = actions[key](p);
                    state.dispatch(action);
                    return action;
                };
                return acc;
            }, {});
            spy = Sinon.spy();
            state.subscribeAction(spy);
            timer = Sinon.useFakeTimers();
        });
        afterEach(() => {
            spy.resetHistory();
            timer.restore();
            state.unsubscribe();
        });
        it('should be able to dispatch reducer action by actions dispatcher #void', () => {
            const action = actionsDispatcher.addOne();
            const [[arg]] = spy.args;
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count + 1);
        });
        it('should be able to dispatch reducer action by actions dispatcher #param', () => {
            const newCount = 100;
            const action = actionsDispatcher.setCount(newCount);
            const [[arg]] = spy.args;
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(newCount);
        });
        it('should be able to dispatch immer reducer action by actions dispatcher #void', () => {
            const action = actionsDispatcher.addCountOneImmer();
            const [[arg]] = spy.args;
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count + 1);
        });
        it('should be able to dispatch immer reducer action by actions dispatcher #param', () => {
            const newCount = new Date();
            const action = actionsDispatcher.setCountImmer(newCount);
            const [[arg]] = spy.args;
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(newCount.valueOf());
        });
        it('should be able to dispatch epic action by actions dispatcher #void', () => {
            const action = actionsDispatcher.asyncAddCountOne();
            const [[arg]] = spy.args;
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count);
            timer.tick(TIME_TO_DELAY);
            const [, [arg1]] = spy.args;
            expect(arg1).toStrictEqual(counter.getActions().setCount(counter.defaultState.count + 1));
            expect(spy.callCount).toBe(2);
        });
        it('should be able to dispatch epic action by actions dispatcher #param', () => {
            const newCount = 100;
            const action = actionsDispatcher.asyncAddCount(newCount);
            const [[arg]] = spy.args;
            expect(arg).toStrictEqual(action);
            expect(spy.callCount).toBe(1);
            expect(state.getState().count).toBe(counter.defaultState.count);
            timer.tick(TIME_TO_DELAY);
            const [, [arg1]] = spy.args;
            expect(arg1).toStrictEqual(counter.getActions().asyncAddCountString(`${newCount}`));
            expect(spy.callCount).toBe(2);
        });
        it('should be able to dispose by other actions', () => {
            const newCount = 100;
            actionsDispatcher.asyncAddCount(newCount);
            expect(spy.callCount).toBe(1);
            actionsDispatcher.dispose$();
            timer.tick(TIME_TO_DELAY);
            expect(spy.callCount).toBe(2);
            expect(state.getState()).toStrictEqual(counter.defaultState);
        });
        it('should be able to dispatch noop action', () => {
            const action = actionsDispatcher.effectWithPureSideEffect();
            const [[arg1], [arg2]] = spy.args;
            expect(arg1).toStrictEqual(action);
            expect(typeof arg2.type).toBe('symbol');
            expect(spy.callCount).toBe(2);
            expect(state.getState()).toStrictEqual(counter.defaultState);
        });
    });
});
