import { __decorate, __metadata } from "tslib";
import 'reflect-metadata';
import { rootInjector } from '@asuka/di';
import * as Sinon from 'sinon';
import { empty, Observable } from 'rxjs';
import { delay, map } from 'rxjs/operators';
import { createState } from '../state';
import { Ayanami } from '../ayanami';
import { Reducer, Effect } from '../decorators';
import { Module } from '../module.decorator';
describe('Smoking tests', () => {
    it('Module should be able to work without effects', () => {
        const { stateCreator } = createState((state, action) => {
            if (action.type === 'foo') {
                return { ...state, name: action.payload };
            }
            return state;
        }, () => empty());
        const state = stateCreator({ name: 'bar' });
        const action = {
            type: 'foo',
            payload: 'foo',
            state,
        };
        state.dispatch(action);
        expect(state.getState().name).toBe(action.payload);
    });
    it('should be able to dispatch actions from the other module', () => {
        const asyncTimeToDelay = 2000;
        const timer = Sinon.useFakeTimers();
        let FooModule = class FooModule extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {
                    foo: '1',
                };
            }
            set(state, payload) {
                return { ...state, foo: payload };
            }
        };
        __decorate([
            Reducer(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object, String]),
            __metadata("design:returntype", void 0)
        ], FooModule.prototype, "set", null);
        FooModule = __decorate([
            Module('Foo')
        ], FooModule);
        let BarModule = class BarModule extends Ayanami {
            constructor(fooModule) {
                super();
                this.fooModule = fooModule;
                this.defaultState = {};
            }
            asyncSetFoo(payload$) {
                return payload$.pipe(delay(asyncTimeToDelay), map((payload) => this.fooModule.getActions().set(payload)));
            }
        };
        __decorate([
            Effect(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Observable]),
            __metadata("design:returntype", void 0)
        ], BarModule.prototype, "asyncSetFoo", null);
        BarModule = __decorate([
            Module('Bar'),
            __metadata("design:paramtypes", [FooModule])
        ], BarModule);
        const fooModuleState = rootInjector.getInstance(FooModule);
        const barModuleState = rootInjector.getInstance(BarModule);
        const fooState = fooModuleState.createState();
        const barState = barModuleState.createState();
        const payload = 'whatever';
        const action = barModuleState.getActions().asyncSetFoo(payload);
        expect(action.payload).toBe(payload);
        barState.dispatch(action);
        timer.tick(asyncTimeToDelay);
        expect(fooState.getState().foo).toBe(payload);
        timer.restore();
        rootInjector.reset();
    });
});
