import { Subject, noop, ReplaySubject, Subscription, identity } from 'rxjs';
import { TERMINATE_ACTION } from '../ssr/terminate';
import { logStateAction } from '../redux-devtools-extension';
import { StateInterface } from './symbols';
export function createState(reducer, effect) {
    const action$ = new Subject();
    const _action$ = new Subject();
    const actionObservers = new Set();
    const state$ = new ReplaySubject(1);
    function stateCreator(defaultState, middleware = identity, loadFromSSR = false) {
        const state = Object.create(null);
        let appState = defaultState;
        function dispatch(action) {
            if (action.state !== state && action.type !== TERMINATE_ACTION.type) {
                action.state.dispatch(action);
                return;
            }
            const prevState = appState;
            const newState = reducer(prevState, action);
            if (newState !== prevState) {
                state$.next(newState);
            }
            logStateAction(action);
            action$.next(action);
            _action$.next(action);
        }
        const effect$ = effect(_action$, loadFromSSR);
        const subscription = new Subscription();
        subscription.add(middleware(effect$).subscribe((action) => {
            try {
                dispatch(action);
            }
            catch (e) {
                action$.error(e);
            }
        }, (err) => {
            console.error(err);
        }));
        subscription.add(action$.subscribe((action) => {
            for (const observer of actionObservers) {
                observer(action);
            }
        }, (err) => {
            _action$.error(err);
        }, () => {
            _action$.complete();
        }));
        subscription.add(state$.subscribe((state) => {
            appState = state;
        }));
        state$.next(defaultState);
        Object.assign(state, {
            [StateInterface]: state,
            dispatch,
            state$,
            getState: () => appState,
            subscribeAction: (observer) => {
                actionObservers.add(observer);
                return () => actionObservers.delete(observer);
            },
            unsubscribe: () => {
                action$.complete();
                state$.complete();
                subscription.unsubscribe();
                state.dispatch = noop;
                actionObservers.clear();
            },
        });
        return state;
    }
    return { stateCreator, action$, state$ };
}
