/* istanbul ignore next */
import { noop } from 'rxjs';
import { TERMINATE_ACTION } from './ssr/terminate';
let devtool = {
    send: noop,
    init: noop,
};
const FakeReduxDevTools = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    connect: () => devtool,
};
export const INIT_ACTION_TYPE = 'INIT_AYANAMI_STATE';
const ReduxDevTools = (typeof window !== 'undefined' && window.__REDUX_DEVTOOLS_EXTENSION__) ??
    FakeReduxDevTools;
const STATE = {};
export let logStateAction = (action) => {
    if (action.type === TERMINATE_ACTION.type) {
        return;
    }
    const namespace = action.state.name;
    const _action = {
        type: `${namespace}/${String(action.type)}`,
        params: filterParams(action.payload),
    };
    STATE[namespace] = action.state.getState();
    if (!action.type?.endsWith(INIT_ACTION_TYPE)) {
        devtool.send(_action, STATE);
    }
};
if (process.env.NODE_ENV !== 'development') {
    logStateAction = noop;
}
export const initDevtool = () => {
    if (process.env.NODE_ENV === 'development') {
        devtool = ReduxDevTools.connect({
            name: `Ayanami`,
        });
        devtool.init(STATE);
    }
};
function filterParams(params) {
    if (params && typeof params === 'object' && typeof Event !== 'undefined') {
        if (params instanceof Event) {
            return `<<Event:${params.type}>>`;
        }
        else if (params.nativeEvent instanceof Event) {
            return `<<SyntheticEvent:${params.nativeEvent.type}>>`;
        }
    }
    return params;
}
