import { __decorate, __metadata } from "tslib";
import 'reflect-metadata';
import React from 'react';
import { Observable, timer } from 'rxjs';
import { endWith, switchMap, map, mergeMap, flatMap, withLatestFrom } from 'rxjs/operators';
import { rootInjector } from '@asuka/di';
import { renderToString } from 'react-dom/server';
import { create, act } from 'react-test-renderer';
import uniqueId from 'lodash/uniqueId';
import { TERMINATE_ACTION, emitSSREffects, SSREffect } from '../index';
import { Ayanami, ImmerReducer, Module, Effect, Reducer } from '../../core';
import { useAyanami, useAyanamiState } from '../../hooks';
import { GLOBAL_KEY } from '../constants';
import { SSRSharedContext, SSRContext } from '../ssr-context';
import { SSRStateCacheInstance } from '../ssr-states';
let CountModel = class CountModel extends Ayanami {
    constructor() {
        super(...arguments);
        this.defaultState = { count: 0, name: '' };
    }
    setCount(state, count) {
        state.count = count;
    }
    setName(state, name) {
        state.name = name;
    }
    getCount(payload$) {
        return payload$.pipe(flatMap(() => timer(20).pipe(map(() => this.getActions().setCount(1)), endWith(TERMINATE_ACTION))));
    }
    skippedEffect(payload$) {
        return payload$.pipe(switchMap((name) => timer(20).pipe(map(() => this.getActions().setName(name)), endWith(TERMINATE_ACTION))));
    }
};
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Number]),
    __metadata("design:returntype", void 0)
], CountModel.prototype, "setCount", null);
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", void 0)
], CountModel.prototype, "setName", null);
__decorate([
    SSREffect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], CountModel.prototype, "getCount", null);
__decorate([
    SSREffect({
        payloadGetter: (ctx, skip) => ctx.url || skip(),
        skipFirstClientDispatch: false,
    }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], CountModel.prototype, "skippedEffect", null);
CountModel = __decorate([
    Module('CountModel')
], CountModel);
let TipModel = class TipModel extends Ayanami {
    constructor() {
        super(...arguments);
        this.defaultState = { tip: '' };
    }
    setTip(state, tip) {
        state.tip = tip;
    }
    getTip(payload$) {
        return payload$.pipe(mergeMap(() => timer(1).pipe(map(() => this.getActions().setTip('tip')), endWith(TERMINATE_ACTION))));
    }
};
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", void 0)
], TipModel.prototype, "setTip", null);
__decorate([
    SSREffect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", Observable)
], TipModel.prototype, "getTip", null);
TipModel = __decorate([
    Module('TipModel')
], TipModel);
const Component = () => {
    const [state, actions] = useAyanami(CountModel);
    React.useEffect(() => {
        actions.setName('new name');
    }, []);
    return (React.createElement(React.Fragment, null,
        React.createElement("span", null, state.count)));
};
const ComponentWithSelector = () => {
    const [state, actions] = useAyanami(CountModel, {
        selector: (s) => ({
            count: s.count + 1,
        }),
    });
    React.useEffect(() => {
        actions.setName('new name');
    }, []);
    return (React.createElement(React.Fragment, null,
        React.createElement("span", null, state.count)));
};
describe('SSR specs:', () => {
    beforeEach(() => {
        SSRStateCacheInstance.setPoolSize(100);
        rootInjector.addProviders([CountModel, TipModel]);
    });
    afterEach(() => {
        SSRStateCacheInstance.setPoolSize(0);
        rootInjector.reset();
    });
    it('should throw if module name not given', () => {
        function generateException() {
            let ErrorModel = class ErrorModel extends Ayanami {
                constructor() {
                    super(...arguments);
                    this.defaultState = {};
                }
            };
            ErrorModel = __decorate([
                (Module())
            ], ErrorModel);
            return ErrorModel;
        }
        expect(generateException).toThrow();
    });
    it('should pass valid module name', () => {
        let Model = class Model extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {};
            }
        };
        Model = __decorate([
            Module('1')
        ], Model);
        let Model2 = class Model2 extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {};
            }
        };
        Model2 = __decorate([
            Module('2')
        ], Model2);
        function generateException1() {
            let ErrorModel1 = class ErrorModel1 extends Ayanami {
                constructor() {
                    super(...arguments);
                    this.defaultState = {};
                }
            };
            ErrorModel1 = __decorate([
                Module('1')
            ], ErrorModel1);
            return ErrorModel1;
        }
        function generateException2() {
            let ErrorModel2 = class ErrorModel2 extends Ayanami {
                constructor() {
                    super(...arguments);
                    this.defaultState = {};
                }
            };
            ErrorModel2 = __decorate([
                Module('1')
            ], ErrorModel2);
            return { ErrorModel2 };
        }
        function generateException3() {
            let ErrorModel = class ErrorModel extends Ayanami {
                constructor() {
                    super(...arguments);
                    this.defaultState = {};
                }
            };
            ErrorModel = __decorate([
                (Module())
            ], ErrorModel);
            return ErrorModel;
        }
        expect(Model).not.toBe(undefined);
        expect(Model2).not.toBe(undefined);
        expect(generateException1).toThrow();
        expect(generateException2).toThrow();
        expect(generateException3).toThrow();
    });
    it('should run ssr effects', async () => {
        const state = await emitSSREffects({ url: 'name' }, [CountModel]);
        const moduleState = state['CountModel'];
        expect(moduleState).not.toBe(undefined);
        expect(moduleState.count).toBe(1);
        expect(moduleState.name).toBe('name');
        expect(state).toMatchSnapshot();
    });
    it('should skip effect if it returns SKIP_SYMBOL', async () => {
        const state = await emitSSREffects({}, [CountModel]);
        const moduleState = state['CountModel'];
        expect(moduleState.name).toBe('');
        expect(state).toMatchSnapshot();
    });
    it('should return right state in hooks', async () => {
        const req = {};
        await emitSSREffects(req, [CountModel]);
        const html = renderToString(React.createElement(SSRContext.Provider, { value: req },
            React.createElement(Component, null)));
        expect(html).toContain('<span>1</span>');
        expect(html).toMatchSnapshot();
    });
    it('should restore state from global', () => {
        // @ts-ignore
        global[GLOBAL_KEY] = {
            CountModel: {
                count: 1,
                name: '',
            },
        };
        const testRenderer = create(React.createElement(Component, null));
        act(() => {
            testRenderer.update(React.createElement(Component, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('1');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should restore state from global #with selector', () => {
        // @ts-ignore
        global[GLOBAL_KEY] = {
            CountModel: {
                count: 10,
                name: '',
            },
        };
        const testRenderer = create(React.createElement(ComponentWithSelector, null));
        act(() => {
            testRenderer.update(React.createElement(ComponentWithSelector, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('10');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should not restore state from global if state is null', () => {
        // @ts-ignore
        global[GLOBAL_KEY] = {
            OtherModule: {
                count: 10,
                name: '',
            },
        };
        const testRenderer = create(React.createElement(ComponentWithSelector, null));
        act(() => {
            testRenderer.update(React.createElement(ComponentWithSelector, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('1');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should restore and skip first action on client side', () => {
        const Component = () => {
            const [state, actions] = useAyanami(CountModel);
            React.useEffect(() => {
                actions.getCount();
            }, []);
            return (React.createElement(React.Fragment, null,
                React.createElement("span", null, state.count)));
        };
        // @ts-ignore
        global[GLOBAL_KEY] = {
            CountModel: {
                count: 2,
                name: '',
            },
        };
        const testRenderer = create(React.createElement(Component, null));
        act(() => {
            testRenderer.update(React.createElement(Component, null));
        });
        expect(testRenderer.root.findByType('span').children[0]).toBe('2');
        // @ts-ignore
        delete global[GLOBAL_KEY];
        testRenderer.unmount();
    });
    it('should support concurrency', async () => {
        return Promise.all([
            emitSSREffects({ url: 'name1' }, [CountModel, TipModel], 'concurrency1'),
            emitSSREffects({ url: 'name2' }, [CountModel, TipModel], 'concurrency2'),
        ]).then(([result1, result2]) => {
            expect(result1['CountModel'].name).toBe('name1');
            expect(result2['CountModel'].name).toBe('name2');
            expect({
                firstRequest: result1,
                secondRequest: result2,
            }).toMatchSnapshot();
        });
    });
    it('should timeout', async () => {
        const req = {};
        const reqContext = uniqueId();
        return emitSSREffects(req, [CountModel], reqContext, 0).catch((e) => {
            expect(e.message).toBe('Terminate timeout');
        });
    });
    it('should resolve empty object if no modules provided', async () => {
        const req = {};
        const state = await emitSSREffects(req, []);
        expect(state).toStrictEqual({});
    });
    it('should do nothing if Module contains no SSREffects', async () => {
        const req = {};
        let WithoutSSRModule = class WithoutSSRModule extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {
                    count: 0,
                };
            }
            set(state, payload) {
                state.count = payload;
            }
            addOne(payload$) {
                return payload$.pipe(withLatestFrom(this.state$), map(([, { count }]) => this.getActions().set(count + 1)));
            }
        };
        __decorate([
            ImmerReducer(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object, Number]),
            __metadata("design:returntype", void 0)
        ], WithoutSSRModule.prototype, "set", null);
        __decorate([
            Effect(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Observable]),
            __metadata("design:returntype", Observable)
        ], WithoutSSRModule.prototype, "addOne", null);
        WithoutSSRModule = __decorate([
            Module('WithoutSSR')
        ], WithoutSSRModule);
        const state = await emitSSREffects(req, [WithoutSSRModule]);
        expect(state).toStrictEqual({});
    });
    it('should throw error if runEffects error', async () => {
        const req = {};
        const error = new TypeError('whatever');
        let SSRErrorModule = class SSRErrorModule extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {
                    count: 0,
                };
            }
            addOne(payload$) {
                return payload$.pipe(withLatestFrom(this.state$), map(() => {
                    throw error;
                }));
            }
        };
        __decorate([
            SSREffect(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Observable]),
            __metadata("design:returntype", Observable)
        ], SSRErrorModule.prototype, "addOne", null);
        SSRErrorModule = __decorate([
            Module('ErrorModule')
        ], SSRErrorModule);
        try {
            await emitSSREffects(req, [SSRErrorModule]);
            throw new TypeError('Unreachable code path');
        }
        catch (e) {
            expect(e).toBe(error);
        }
    });
    it('should throw error if reducer throw', async () => {
        const req = {};
        const error = new TypeError('whatever');
        let SSRErrorModule = class SSRErrorModule extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {
                    count: 0,
                };
            }
            set() {
                throw error;
            }
            addOne(payload$) {
                return payload$.pipe(withLatestFrom(this.state$), map(() => this.getActions().set()));
            }
        };
        __decorate([
            Reducer(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", void 0)
        ], SSRErrorModule.prototype, "set", null);
        __decorate([
            SSREffect(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Observable]),
            __metadata("design:returntype", Observable)
        ], SSRErrorModule.prototype, "addOne", null);
        SSRErrorModule = __decorate([
            Module('ErrorReducerModule')
        ], SSRErrorModule);
        try {
            await emitSSREffects(req, [SSRErrorModule]);
            throw new TypeError('Unreachable code path');
        }
        catch (e) {
            expect(e).toBe(error);
        }
    });
    it('should throw error if payloadGetter throw', async () => {
        const req = {};
        const error = new TypeError('whatever');
        let SSRErrorModule = class SSRErrorModule extends Ayanami {
            constructor() {
                super(...arguments);
                this.defaultState = {
                    count: 0,
                };
            }
            set(state, payload) {
                return { ...state, count: payload };
            }
            addOne(payload$) {
                return payload$.pipe(withLatestFrom(this.state$), map(([, state]) => this.getActions().set(state.count + 1)));
            }
        };
        __decorate([
            Reducer(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object, Number]),
            __metadata("design:returntype", void 0)
        ], SSRErrorModule.prototype, "set", null);
        __decorate([
            SSREffect({
                payloadGetter: () => {
                    throw error;
                },
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Observable]),
            __metadata("design:returntype", Observable)
        ], SSRErrorModule.prototype, "addOne", null);
        SSRErrorModule = __decorate([
            Module('ErrorMiddlewareModule')
        ], SSRErrorModule);
        try {
            await emitSSREffects(req, [SSRErrorModule]);
            throw new TypeError('Unreachable code path');
        }
        catch (e) {
            expect(e).toBe(error);
        }
    });
    it('should reuse state if provider context', async () => {
        const requestId = uniqueId();
        const req1 = {};
        const req2 = {};
        await emitSSREffects(req1, [CountModel], requestId);
        await emitSSREffects(req2, [CountModel], requestId);
        const SharedComponent1 = () => {
            const state = useAyanamiState(CountModel);
            return (React.createElement(React.Fragment, null,
                React.createElement("span", null, state.count)));
        };
        const SharedComponent2 = () => {
            const state = useAyanamiState(CountModel);
            return (React.createElement(React.Fragment, null,
                React.createElement("span", null, state.count)));
        };
        const cachedState = SSRStateCacheInstance.get(requestId, CountModel);
        expect(cachedState.getState().count).toBe(1);
        const result1 = renderToString(React.createElement(SSRSharedContext.Provider, { value: requestId },
            React.createElement(SharedComponent1, null)));
        const result2 = renderToString(React.createElement(SSRSharedContext.Provider, { value: requestId },
            React.createElement(SharedComponent2, null)));
        expect(result1).toBe(result2);
    });
});
