import { SSRSymbol, ACTION_TO_SKIP_KEY } from './constants';
import { Effect } from '../core/decorators';
export const SKIP_SYMBOL = Symbol('skip');
function addDecorator(target, method, middleware) {
    const existedMetas = Reflect.getMetadata(SSRSymbol, target);
    const meta = { action: method, middleware };
    if (existedMetas) {
        existedMetas.push(meta);
    }
    else {
        Reflect.defineMetadata(SSRSymbol, [meta], target);
    }
}
export function SSREffect(options) {
    const { payloadGetter, skipFirstClientDispatch } = {
        payloadGetter: undefined,
        skipFirstClientDispatch: true,
        ...options,
    };
    return (target, method, descriptor) => {
        addDecorator(target, method, payloadGetter);
        if (skipFirstClientDispatch) {
            const actionsToSkip = Reflect.getMetadata(ACTION_TO_SKIP_KEY, target) || new Set();
            actionsToSkip.add(method);
            Reflect.defineMetadata(ACTION_TO_SKIP_KEY, actionsToSkip, target);
        }
        return Effect()(target, method, descriptor);
    };
}
