import { StateInterface } from '../core/symbols';
export class SSROneShotCache {
    constructor() {
        this.cache = new Map();
        this.activedCtx = new Set();
    }
    store(ctx, ctor, instance) {
        if (this.cache.has(ctx)) {
            this.cache.get(ctx).set(ctor, instance);
        }
        else {
            const cache = new Map();
            cache.set(ctor, instance);
            this.cache.set(ctx, cache);
        }
    }
    consume(ctx, ctor) {
        const instance = this.cache.get(ctx)?.get(ctor);
        if (!this.activedCtx.has(ctx) && instance) {
            this.activedCtx.add(ctx);
            Promise.resolve().then(() => {
                this.destroy(ctx);
            });
        }
        return instance;
    }
    destroy(ctx) {
        if (this.activedCtx.has(ctx)) {
            this.activedCtx.delete(ctx);
            if (this.cache.has(ctx)) {
                for (const instance of this.cache.get(ctx).values()) {
                    if (instance && instance[StateInterface]) {
                        instance[StateInterface].unsubscribe();
                    }
                }
                this.cache.delete(ctx);
            }
        }
    }
}
export const oneShotCache = new SSROneShotCache();
