export class SSRStateCache {
    constructor() {
        this.SSRStates = new Map();
        this.size = 1000;
        this.cleanupStateByContext = (ctx) => {
            const states = this.SSRStates.get(ctx);
            if (states) {
                for (const [, state] of states) {
                    state.unsubscribe();
                }
            }
            this.SSRStates.delete(ctx);
        };
    }
    setPoolSize(size) {
        if (this.SSRStates.size >= size) {
            this.SSRStates.clear();
        }
        this.size = size;
    }
    has(ctx, constructor) {
        return this.SSRStates.has(ctx) && this.SSRStates.get(ctx).has(constructor);
    }
    get(ctx, constructor) {
        return this.SSRStates.get(ctx)?.get(constructor);
    }
    set(ctx, constructor, state) {
        if (this.SSRStates.size >= this.size) {
            this.SSRStates.clear();
        }
        if (this.SSRStates.has(ctx)) {
            this.SSRStates.get(ctx).set(constructor, state);
        }
        else {
            const caches = new Map();
            caches.set(constructor, state);
            this.SSRStates.set(ctx, caches);
        }
    }
}
export const SSRStateCacheInstance = new SSRStateCache();
